#include "elastortomat.h"
#include "global.h"
#include "intpoints.h"
#include "elemswitch.h"
#include "vecttens.h"



elastortomat::elastortomat (void)
{
  allocm(6, 6, de);
  allocm(3, 3, t);
}



elastortomat::~elastortomat (void)
{
}



/**
  Function reads material parameters from the opened text file.
   
  @param in - pointer to the opened XFILE

  @return The function does not return anything.

  Created by Tomas Koudelka, 19.12.2014
*/
void elastortomat::read (XFILE *in)
{
  xfscanf(in, "%k", "ortomat");
  readm(in, de);
  xfscanf(in, "%k", "transfmat");
  readm(in, t);
}



/**
  Function prints material parameters into the opened text file.
   
  @param out - pointer to the opened FILE

  @return The function does not return anything.

  Created by Tomas Koudelka, 19.12.2014
*/
void elastortomat::print (FILE *out)
{
  printm(de, out);
  printm(out, t);
}



/**
  The function assembles the stiffness %matrix in the global
  coordinate system.

  @param d - resulting stiffness %matrix
  @param ipp - integration point id

  @return The funtion returns resulting %matrix in the argument d.

  Created by Tomas Koudelka, 19.12.2014
*/
void elastortomat::matstiff (matrix &d, long ipp)
{
  matrix tmat(3,3), dg(6,6);
  vector p(3);
  char *namevar[] = {"x", "y", "z"};

  // x,y,z coordinates of the integration point will be stored in the vector p
  ipcoord(Mm->elip[ipp], ipp, 0, 0, p);
  // evaluate transformation matrix
  t.evaluate(p, namevar, tmat);
  // transformation to the global coordinate system
  lg_tens4transf(dg, de, tmat, Mm->ip[ipp].ssst);
  // convert stiffness matrix(6,6) to reduced form depending on ssst
  tensor4_matrix(d, dg, Mm->ip[ipp].ssst);
}



/**
  The function assembles the stiffness %matrix in the local
  coordinate system.

  @param d - resulting stiffness %matrix
  @param ssst - stress/strain state indicator

  @return The funtion returns resulting %matrix in the argument d.

  Created by Tomas Koudelka, 19.12.2014
*/
void elastortomat::loc_matstiff (matrix &d, strastrestate ssst)
{
  copym(de, d);
//  tensor4_matrix (d, de, ssst);
}



/** 
  The function returns transformation %matrix from the local material coordinate system 
  to the global one in the argument. Components of the transformation %matrix
  are evaluated with respect to space coordinates of the given integration point ipp.
  The argument tmat is supposed to be allocated to dimensions 3x3.

  @param tmat - evaluated transformation %matrix 
  @param ipp - integration point id

  @return The function returns transformation %matrix in the argument tmat

  Created by Tomas Koudelka, 19.12.2014
*/
void elastortomat::give_transf_mat (matrix &tmat, long ipp)
{
  vector p(3);
  char *namevar[] = {"x", "y", "z"};

  // x,y,z coordinates of the integration point will be stored in the vector p
  ipcoord(Mm->elip[ipp], ipp, 0, 0, p);
  // evaluate transformation matrix
  t.evaluate(p, namevar, tmat);
}



/**
  The function computes stresses for the given strains.
  Resulting stresses are given in the global coordinate system.

  @param ipp - integration point id

  @return The funtion stores the resulting stresses in the stress array 
          of the given integration point.

  Created by Tomas Koudelka, 19.12.2014
*/
void elastortomat::nlstresses (long ipp)
{
  long i, n = Mm->ip[ipp].ncompstr;
  vector eps(n),sig(n);
  matrix d(n,n);
  
  //  initial values
  for (i=0;i<n;i++){
    eps[i]=Mm->ip[ipp].strain[i];
  }
  
  // sig = D.eps
  matstiff (d,ipp);
  mxv (d,eps,sig);

  for (i=0;i<n;i++){
    Mm->ip[ipp].stress[i]=sig[i];
  }
}
