#include "mpi.h"
#include <math.h>
#include <string.h>
#include <stdlib.h>
#include "pglobal.h"
#include "pnewtonraph.h"
#include "nssolver.h"
#include "backupsol.h"
#include "global.h"
#include "globmat.h"
#include "mechprint.h"
#include "gmatrix.h"
#include "gtopology.h"
#include "mechprint.h"
#include "mathem.h"
#include "vector.h"
#include "matrix.h"




/**
  Function performs calculation of one parallel load/time step of the Newton-Raphson 
  method for the given load case. Solved equation system does not contain 
  time variable.
  
  @param lcid  - load case id
  @param nlman - pointer to structure conatining setup of the solver
  @param ra    - %vector of attained displacements
  @param fa    - attained load %vector
  @param fb    - residual %vector or load %vector increment - right-hand side %vector
  @param dr    - %vector of displacement increment - left-hand side %vector
  @param istep - time/load step id
  @param j     - inner loop step id (output parameter, set to -1 if no inner loop was performed) 
  @param li    - initial value of time/load step id
  @param ierr  - required normed error of residual %vector

  @return The function returns reached lambda parameter.

  TKr, 19/02/2013 according to Tomas Koudelka
*/
double par_gnewton_raphson_one_step(long lcid, nonlinman *nlman, double *fa, double *ra, double *fb, double *dr, double *fi, 
                                long istep, long &j, long li, long ini, double ierr)
{
  long n = Ndofm;
  double norf, norfa;
  matrix lsm_a(3,3); // least square matrix for the divergence detection
  vector lsm_r(3), lsm_l(3); // right hand and left hand side for the divergence detection
  
  //  assembling of tangent stiffness matrix
  assemble_stiffness_matrix(lcid,istep,-1,li,no);
    
  //  solution of K(r).v=F
  Psolm->par_linear_solver (Gtm,Smat,dr,fb,Out,Mespr);//Mp->ssle->solve_system (Gtm,Smat,dr,fb,Out);

  //  update total displacement vector
  //  ra[j]+=dr[j];
  addv(ra, dr, n);
    
  //  computation of internal forces
  internal_forces (lcid,fi);
    
  //  vector of unbalanced forces
  //  fb[j] = fa[j] - fi[j];
  subv(fa, fi, fb, n);

  // norm of attained load vector
  norfa=Psolm->pss (fa,fa,Out);//normv(fa,n);
  //  norm of vector of unbalanced forces
  norf=Psolm->pss(fb,fb,Out);//normv(fb,n);
  if (norfa != 0.0)
    norf /= norfa;
    
  if ((Myrank==0)&&(Mespr==1)) 
  fprintf (stdout,",    norm %e",norf);

  if (norf<ierr)
  {
    j = -1;
    return norf;
  }
    
  //  internal iteration loop
  fillm(0.0, lsm_a);
  fillv(0.0, lsm_r);
  for (j=0; j<ini; j++)
  {
    Mp->jstep = j;

    // re-assemble stifness matrix if it is required
    assemble_stiffness_matrix(lcid,istep,j,li,no);
    
    // solution of K(r).v=F
    Psolm->par_linear_solver (Gtm,Smat,dr,fb,Out,Mespr);//Mp->ssle->solve_system(Gtm,Smat,dr,fb,Out);
    
    // ra[k]+=dr[k];
    addv(ra, dr, n);
      
    // computation of internal forces
    internal_forces(lcid,fi);
      
    // vector of unbalanced forces
    // fb[k]=fa[k]-fi[k]
    subv(fa, fi, fb, n);
      
    // norm of vector of unbalanced forces
    norf=Psolm->pss(fb,fb,Out);//normv(fb,n);
    if (norfa != 0.0)
      norf /= norfa;

    if ((Myrank==0) && (Mespr==1))
      fprintf (stdout,"\n increment %ld   inner loop j %ld     norf %e",istep,j,norf);
      
    if (norf < ierr) 
      // equilibrium was attained
      return norf;

    // divergence detection with help of the least square method
    if (check_divergency(nlman, lsm_a, lsm_r, lsm_l, j, norf))
      return norf;
  }

  // equilibrium was not attained in this step
  return norf;  
}
