#include "ppcsolver.h"

#include "global.h"
#include "globalt.h"
#include "globalc.h"
#include "pglobalc.h"

#include "globmat.h"
#include "globmatt.h"
#include "globmatc.h"

#include "mechprint.h"
#include "transprint.h"

/**
   function solves partially coupled thermo-hydro-mechanical problem in parallel
   transport problems are linear or nonlinear
   mechanical analysis is always nonlinear
   
   JK
*/
void par_solve_pcouplprob ()
{
  long lcid;
  
  //  load case id must be equal to zero
  //  the problems are nonlinear and superposition method cannot be used
  lcid=0;
  
  switch (Cp->tnlinsol){
  case newtonc:{
    par_newton_raphson_parcoupl (lcid);
    break;
  }
  default:{
    print_err("unknown solver of nonlinear equation system is required",__FILE__,__LINE__,__func__);
  }
  }
  
}


/**
   function solves partially coupled thermo-hydro-mechanical time-dependent problem
   by the Newton-Raphson method
   
   @param lcid - load case id
   
   JK, 5.8.2008
*/
void par_newton_raphson_parcoupl (long lcid)
{
  switch (Tp->tprob){
  case nonstationary_problem:{
    //  linear nonstationary transport problem
    //  nonlinear mechanical problem
    par_newton_raphson_parcoupl_lin_vform (lcid);
    break;
  }
  case nonlinear_nonstationary_problem:{
    //  nonlinear nonstationary transport problem
    //  nonlinear mechanical problem
    par_newton_raphson_parcoupl_nonlin (lcid);
    break;
  }
  default:{
    print_err("unknown TRFEL problem type is required",__FILE__,__LINE__,__func__);
  }
  }
}

/**
   function solves partially coupled problems in parallel,
   transport processes are linear, mechanical analysis is nonlinear
   systems of nonlinear equations are solved by the Newton-Raphson method
   trapezoidal rule is in the v-form, it means, nodal time derivatives are calculated
   nodal values are computed later from time derivatives
   
   @param lcid - load case id
   
   JK, 19.5.2004, 11.6.2008
*/
void par_newton_raphson_parcoupl_lin_vform (long lcid)
{
  long i,j,k,nm,nt,ini,nii,nsts;
  double zero,dt,dtmin,dtdef,end_time,alpha,norfa,norf,err,s;
  double *r,*dr,*d,*f,*fb,*fp,*fi,*fl,*p,*lhsb,*lhst,*lhstb,*tdlhst,*rhst;
  matrix lsm_a(3,3);
  vector lsm_r(3), lsm_l(3);
  MPI_Status stat;
  char file[FNAMELEN+20];
  FILE *aux;
  char *path;
  char *name;
  char *suffix;
  
  
  //  number of mechanical degrees of freedom
  nm=Ndofm;
  //  number of transport degrees of freedom
  nt=Ndoft;
  
  //Mp->filename_decomposition (Outdm->outfn,path,name,suffix);
  //sprintf (file,"%s.bac",Outdm->outfn);
  //aux = fopen ("zaloha","w");
  
  
  // *************************** 
  //  vectors of transport part  
  // *************************** 
  
  //  vector of nodal values
  lhst=Lsrst->give_lhs (lcid);
  //  vector of time derivatives of nodal values
  tdlhst=Lsrst->give_tdlhs (lcid);
  //  vector of the right hand side
  rhst=Lsrst->give_rhs (lcid);
  
  //  array containing predictors
  d = new double [nt];
  nullv (d,nt);
  //  predictor vector
  p = new double [nt];
  nullv (p,nt);
  //  backup of the lhs vector
  lhstb = new double [nt];
  nullv (lhstb,nt);


  // **************************** 
  //  vectors of mechanical part  
  // **************************** 
  
  //  vector of nodal displacements
  r = Lsrs->give_lhs (lcid);
  //  vector of nodal prescribed forces
  f = Lsrs->give_rhs (lcid);
  
  //  vector of increments of nodal displacements
  dr = new double [nm];
  nullv (dr,nm);
  //  vector of prescribed forces from the previous step
  fp = new double [nm];
  nullv (fp,nm);
  //  vector of force loads
  fl = new double [nm];
  nullv (fl,nm);
  //  vector of internal forces
  fi = new double [nm];
  nullv (fi,nm);
  //  auxiliary force vector
  fb = new double [nm];
  nullv (fb,nm);
  //  backup of the nodal displacements
  lhsb = new double [nm];
  nullv (lhsb,nm);


  //  coefficient of trapezoidal method
  alpha=Tp->alpha;
  //  computer zero
  zero=Cp->zero;
  //  maximum number of iterations in inner loop in mechanics
  ini = Mp->niilnr;
  //  required norm of vector of unbalanced forces
  err = Mp->errnr;
  
  
  
  //  starting time
  Cp->time=Cp->timecon.starttime ();
  //  time increment
  dt=Cp->timecon.initialtimeincr ();
  //  end time
  end_time = Cp->timecon.endtime ();
  

  //  mechanical time controller is rewritten by coupled time controller
  Mp->timecon.take_values (Cp->timecon);
  //Mp->timecon.seconds_days ();
  Mp->time=Cp->time;
  
  //  transport time controller is rewritten by coupled time controller
  Tp->timecont.take_values (Cp->timecon);
  Tp->time = Cp->time;
  
  
  //  nodes - integration points interpolation
  approximation ();
  //  initial temperature is approximated to the mechanical integration points
  approximation_inittemper ();
  approximation_humid ();
  
  actual_previous_nodval ();
  
  Gtm->comp_domain_sizes ();
  Mb->alloc_sumcomp ();
  
  
  //  initiation of mechanical material models
  Mm->initmaterialmodels();
  //  initiation of transport material models
  Tm->initmaterialmodels();
  
  compute_req_valt (0);
  
  
  //  number of time step
  i=0;
  //  number of successful time steps (number of steps without inner iterations)
  nsts=0;
  
  // ****************************************
  //  print of initial state of the problem
  // ****************************************
  print_init(-1, "wt",Pcp->fnim,Pcp->feim);
  print_step(0,i,Cp->time,NULL);
  print_flush();
  
  print_initt(-1, "wt",Pcp->fnit,Pcp->feit);
  print_stept(0,i,Cp->time,NULL);
  print_flusht();
  
  
  // **************************
  //  main iteration loop  ***
  // **************************
  
  do{
    
    //  update of time
    Cp->time=Cp->timecon.newtime (dt);
    
    if (Mesprc != 0)
      fprintf (stdout,"\n\n time   %le,   step number %ld",Cp->time,i);
    
    // ********************************
    //  transport part of the problem
    // ********************************
    
    
    //  conductivity matrix
    conductivity_matrix (lcid);
    
    //  capacity matrix
    capacity_matrix (lcid);
    
    //  predictor   d+(1-alpha)*dt*v
    for (j=0;j<nt;j++){
      p[j] = lhst[j] + (1.0-alpha)*dt*tdlhst[j];
    }
    
    //  right hand side of transport part
    trfel_right_hand_side (0,rhst,nt);
    
    //  auxiliary vector
    Kmat->gmxv (d,p);
    
    //  matrix of the system of equations
    //  C + alpha.dt.K
    Kmat->scalgm (dt*alpha);
    Kmat->addgm (1.0,*Cmat);
    
    for (j=0;j<nt;j++){
      rhst[j] = rhst[j] - d[j];
      d[j]=tdlhst[j];
    }
    
    //  solution of the system of algebraic equations
    Psolt->par_linear_solver (Gtt,Kmat,tdlhst,rhst,Outt,Mespr);
    
    //  backup of attained nodal values
    for (j=0;j<nt;j++){
      lhstb[j]=lhst[j];
    }
    
    //  new nodal values
    for (j=0;j<nt;j++){
      s=(1.0-alpha)*d[j]+alpha*tdlhst[j];
      lhst[j]+=dt*s;
    }

    //  physically corrected solution
    solution_correction ();    
    //  approximation of nodal values into ontegration points
    approximation ();

    actual_previous_nodval ();


    
    print_stept(0,i,Cp->time,NULL);
    print_flusht();
    
    
    // *********************************************************
    //  end of transport part and beginning of mechanical part
    // *********************************************************
    
    if (Mp->time <= Tp->time){
      
      //  indicator of strain computation
      //  it means, strains at integration points have not been computed
      Mp->strainstate=0;
      //  indicator of stress computation
      //  it means, stresses at integration points have not been computed
      Mp->stressstate=0;
      //  indicator of computation of other array
      //  it means, stresses at integration points have not been computed
      Mp->otherstate=0;
      
      //  approximation of temperatures from TRFEL nodes to MEFEL integration points
      approximation_temper ();
      //  approximation of humidities from TRFEL nodes to MEFEL integration points
      approximation_humid ();
      
      
      //  backup of attained nodal values
      for (j=0;j<nm;j++){
	lhsb[j]=r[j];
      }
      
      //  assembling of the right hand side (prescribed load)
      mefel_right_hand_side (lcid,f,fl);
      
      //  computation of sums of force components in particular directions
      //  it is used in special creep and consolidation models
      Mb->comp_sum (f);
      
      //  computation of unbalanced forces which will be on the right hand side
      incr_internal_forces (lcid,fi);
      
      for (j=0;j<nm;j++){
	fb[j]=f[j]-fp[j]+fi[j];
	fp[j]=f[j];
      }    
      
      //  assembling of tangent stiffness matrix
      stiffness_matrix (lcid);
      
      //  solution of K(r).v=F
      Psolm->par_linear_solver (Gtm,Smat,dr,fb,Out,Mespr);
      
      //  new displacement vector
      for (j=0;j<nm;j++){
	r[j]+=dr[j];
      }
      
      //  computation of internal forces
      internal_forces (lcid,fi);
      
      norfa=Psolm->pss (f,Out);
      
      for (j=0;j<nm;j++){
	fb[j] = fl[j] - fi[j];
      }
      //  norm of vector of unbalanced forces
      norf=Psolm->pss (fb,Out);
      
      if (norfa > 0.0)
	norf /= norfa;
      
      if (Psolm->compare_on_master(norf, err) <= 0){ // norf <= err
	//  number of successful time steps
	nsts++;
	//  no inner iteration
	nii=1;
      }
      else{
	//  inner iteration is needed
	nii=0;
      }
      
      if (nii==0){
	// iteration of unbalanced forces caused by time independent models
	//  internal iteration loop
	for (j=0;j<ini;j++){
	  
	  if(Cp->fcsolv == fullnewtonc){
	    //cleaning of stiffness matrix
	    if (Smat != NULL){
	      delete Smat;
	      Smat=NULL;
	    }
	    //assembling of tangent stiffness matrix for fullnewton
	    stiffness_matrix (lcid);
	  }
	  
	  //  solution of K(r).v=F
	  Psolm->par_linear_solver (Gtm,Smat,dr,fb,Out,Mespr);
	  
	  for (k=0;k<nm;k++)
	    r[k]+=dr[k];
	  
	  //  computation of internal forces
	  internal_forces (lcid,fi);
	  
	  //  vector of unbalanced forces
	  for (k=0;k<nm;k++)
	    fb[k]=fl[k]-fi[k];
	  
	  //  norm of vector of unbalanced forces
	  norf=Psolm->pss(fb,Out);
	  if (norfa > 0.0)
	    norf /= norfa;
	  
	  if ((Myrank==0) && (Mespr==1))
	    fprintf (stdout,"\n Time increment %lf   inner loop j %ld     norf=%e",Mp->time,j,norf);
	  if (Psolm->compare_on_master(norf, err) <= 0){ // norf <= err
	    //  number of successful time steps
	    nsts++;
	    //  no inner iteration
	    nii=1;
	  }
	  else
	    nii=0;
	  
	  if (nii==1)
	    break;
	}
      }
      
      if (nii==0){
	//  inner iteration was not successful
	//  algorithm has to return to the beginning
	
	//  backup is used for actual values
	for (j=0;j<nm;j++){
	  r[j]=lhsb[j];
	}
	//  reduction of the time increment because
	//  inner loop was not able to enforce equilibrium
	dt/=2.0;
	Mp->timecon.oldtime ();
	i--;
	
	if ((Myrank==0) && (Mespr==1))
	  fprintf (stdout,"\n time increment is reduced because the inner loop was not able to enforce equilibrium");
	if (dt<dtmin){
	  fprintf (stderr,"\n\n time increment is less than minimum time increment");
	  fprintf (stderr,"\n computation fails (file %s, line %d)\n",__FILE__,__LINE__);
	  abort();
	}
	
      }
      else{
	//  inner iteration was successful
	
	dtdef = Mp->timecon.actualforwtimeincr ();
	if (nsts==2){
	  dt*=2.0;
	  nsts=0;
	  if ((Myrank==0) && (Mespr==1))
	    fprintf (stdout,"\n time increment is enlarged because no inner loop was neccessary in previous 3 steps");
	}
	if (dt>dtdef)
	  dt=dtdef;
	
	Mm->updateipval();
	
	compute_req_val (lcid);
	print_step(lcid, i, Mp->time, f);
	print_flush();
      }
      
    }
    
  }while(Cp->time<=end_time);
  
  print_close ();
  print_closet ();
  
  delete [] fi;
  delete [] fb;
  delete [] fp;
  delete [] f;
  delete [] p;
  delete [] d;
  delete [] dr;
  //delete [] r;

  //fclose (aux);

}




/**
   function solves partially coupled problems in parallel,
   transport processes are nonlinear, mechanical analysis is nonlinear
   systems of nonlinear equations are solved by the Newton-Raphson method
   
   @param lcid - load case id
   
   JK, 19.9.2007, 11.6.2008
*/
void par_newton_raphson_parcoupl_nonlin (long lcid)
{
/*
  long i,ii,j,k,nm,nt,ini;
  double zero,dt,end_time,alpha;
  double *dr,*fb,*r,*d,*f,*fp,*fi,*p,*lhst,*tdlhst,*rhst;
  double *fbt,*fit,*lhst_last;
  //double s;
  double norf_last;
  double norf, norfa, err;  
  char fname[1020];
  char *path;
  char *name;
  char *suffix;
  matrix lsm_a(3,3);
  vector lsm_r(3), lsm_l(3);
  FILE *aux;
  MPI_Status stat;
  

  Mp->filename_decomposition (Outdm->outfn,path,name,suffix);
  Mp->path = path;
  Mp->filename=name;
  Mp->suffix=suffix;
  sprintf (fname,"%s%s.bac",path, name);
  aux = fopen (fname,"w");


  //  maximum number of iterations in inner loop
  ini = Cp->niilnr;
  //  required norm of vector of unbalanced forces
  err = Cp->errnr;
  //  number of mechanical degrees of freedom
  nm=Ndofm;
  //  number of transport degrees of freedom
  nt=Ndoft;

  // ***************************
  //  vectors of transport part 
  // ***************************
  //  left hand side
  lhst=Lsrst->give_lhs (0);
  //  time derivatives of nodal values
  tdlhst=Lsrst->give_tdlhs (0);
  //  right hand side
  rhst=Lsrst->give_rhs (0);

  //  array containing predictors
  d = new double [nt];
  //  auxiliary vector
  p = new double [nt];
  fbt = new double [nt];
  fit = new double [nt];
  lhst_last = new double [nt];

  //  initial values
  nullv (lhst,nt);
  nullv (tdlhst,nt);
  nullv (d,nt);
  nullv (p,nt);
  nullv (fbt,nt);
  nullv (fit,nt);
  
  // **************************** 
  //  vectors of mechanical part  
  // **************************** 
  fp = new double [nm];
  fb = new double [nm];
  fi = new double [nm];
  dr = new double [nm];

  //  initial values
  nullv (fp,nm);
  nullv (fb,nm);
  nullv (fi,nm);
  nullv (dr,nm);
 
  //  initialization phase
  r = Lsrs->give_lhs (lcid);
  f = Lsrs->give_rhs (lcid);
  
  //  coefficient of trapezoidal method
  alpha=Tp->alpha;
  zero=Tp->zero;
  
  //  driving time from METR
  //  starting time
  Cp->time=Cp->timecon.starttime ();
  //  time increment
  dt=Cp->timecon.initialtimeincr ();
  //  end time
  end_time = Cp->timecon.endtime ();
  
  
  //  mechanical time controller is rewritten by coupled time controller
  Mp->time = Cp->time;
  Mp->timecon.take_values (Cp->timecon);
  //  transport time controller is rewritten by coupled time controller
  Tp->time = Cp->time;
  Tp->timecont.take_values (Cp->timecon);
  
  //  nodes - integration points interpolation in TRFEL
  //  values at integration points are interpolated from nodal values
  approximation ();
  //  initial temperature is approximated to the mechanical integration points
  approximation_inittemper ();

  approximation_humid ();
  
  //  initiation of mechanical material models
  Mm->initmaterialmodels();
  //  initiation of transport material models
  Tm->initmaterialmodels();

  i=0;
  ii=0;
  print_init(-1, "wt");
  print_step(lcid,ii,Mp->time,f);
  print_flush();
  print_initt(-1, "wt");
  print_stept(0,i,Tp->time,NULL);
  print_flusht();


  // **************************
  //  main iteration loop  ***
  // **************************  

  do{
    
    // *********************************************************
    //                     transport part
    // *********************************************************
    
    if (Mesprc==1)  fprintf (stdout,"\n\n -----------------------------------------------------------------------------");
    if (Mesprc==1)  fprintf (stdout,"\n metr time %e,    trfel time %e,   mefel time %e",Cp->time,Tp->time,Mp->time);
    if (Mesprc==1)  fprintf (stdout,"\n ---------------------------------------------------------------------------\n");

    if (Mesprc==1)  fprintf (stdout,"\n\n provadi se TRFEL,  trfel time %e",Tp->time);
            
    Tm->updateipval ();
    i++;
    
    //  backup of attained nodal values and their time derivatives
    for (j=0;j<n;j++){
      lhsb[j]=lhs[j];
      tdlhsb[j]=tdlhs[j];
    }
    
    //  capacity matrix
    capacity_matrix (lcid);

    fprintf (Outt,"\n matice kapacity");
    Cmat->printmat (Outt);
    
    //  conductivity matrix
    conductivity_matrix (lcid);

    fprintf (Outt,"\n matice vodivosti");
    Kmat->printmat (Outt);
    
    //fprintf (Outt,"\n nejvetsi vl. cislo %le",Kmat->power_method (diag,1000,1.0e-6));
    //fprintf (Outt,"\n nejmensi vl. cislo %le",Kmat->inverse_iteration (diag,1000,1.0e-6));
    //conductivity_matrix (lcid);

    
    //  predictor d
    for (j=0;j<n;j++){
      d[j] = lhs[j]+dt*(1.0-alpha)*tdlhs[j];
    }
    
    //  C.d
    Cmat->gmxv (d,p);
    
    //  matrix of the system of equations
    //  C + alpha.dt.K
    Kmat->scalgm (dt*alpha);
    Kmat->addgm (1.0,*Cmat);
    
    
    //  prescribed fluxes f_{n+1}
    trfel_right_hand_side (lcid,f,n);

    //  C.d + alpha dt f_n+1
    norrhs=0.0;
    for (j=0;j<n;j++){
      rhs[j] = f[j]*alpha*dt + p[j];
	//  fprintf (stdout,"\n rhs   %15.10le   k je %d",rhs[j],j);
      //norrhs+=f[j]*f[j];
      //norrhs+=rhs[j]*rhs[j];
      //z[j]=rhs[j];
    }
    //norrhs=sqrt(norrhs);
    
    //Kmat->printmat (Outt);
    
    //  solution of the system of equations
    //  nodal values d_{n+1} are solved
    

    Kmat->diag_scale (diag);
    for (k=0;k<n;k++){
      rhs[k]*=diag[k];
    }

    Kmat->printmat (Outt);
    
    Psolt->par_linear_solver (Gtt,Kmat,lhs,rhs,Outt,Mespr);
    
    for (k=0;k<n;k++){
      lhs[k]*=diag[k];
    }


    //  computation of time derivatives of the nodal values
    for (j=0;j<n;j++){
      tdlhs[j]=(lhs[j]-d[j])/dt/alpha;
    }

    //  physically corrected solution
    solution_correction ();    
    //  approximation of nodal values into ontegration points
    approximation ();
    
    // nulleqother ();
    actual_previous_nodval ();
    
    
    
    //  iteration for equilibrium
    for (j=0;j<ini;j++){
      
      //  capacity matrix
      capacity_matrix (lcid);
      
      //  conductivity matrix
      conductivity_matrix (lcid);
      
      
      //  matrix of the system of equations
      //  C + alpha.dt.K
      Kmat->scalgm (dt*alpha);
      Kmat->addgm (1.0,*Cmat);
      
      //  (C + alpha.dt.K).d
      Kmat->gmxv (lhs,v);
      
      //  C.d
      Cmat->gmxv (d,p);
      
      //  computation of residuals
      //norrhs=0.0;
      //norres=0.0;
      for (k=0;k<n;k++){
	rhs[k]=f[k]*alpha*dt+p[k]-v[k];
	//	fprintf (stdout,"\n rhs   %15.10le   k je %d",rhs[k],k);
	//norres+=rhs[k]*rhs[k];
	z[k]=f[k]*alpha*dt+p[k];
	//norrhs+=z[k]*z[k];
	//fprintf (stdout,"\n rhs   %15.10le   z  %15.10le  v %15.10le",rhs[k],z[k],v[k]);
      }
      //norres=sqrt(norres);
      //norrhs=sqrt(norrhs);
      
      //fprintf (stdout,"\n KONTROLA norres   %15.10le",norres);
      
      stop = norm_computation2 (rhs,z,err,2,1);
      
      //fprintf (stdout,"\n pauza \n");
      
      if (stop==1){
	break;
      }
      
      if (Mesprt != 0)  fprintf (stdout,"\n iteration number %ld",j);
      
      
      for (k=0;k<n;k++){
	rhs[k]=f[k]*alpha*dt+p[k];
	//z[k]=lhs[k];
      }
      
      Kmat->diag_scale (diag);
      for (k=0;k<n;k++){
	rhs[k]*=diag[k];
      }
      
      
      Psolt->par_linear_solver (Gtt,Kmat,lhs,rhs,Outt,Mespr);
      
      for (k=0;k<n;k++){
	lhs[k]*=diag[k];
      }
      
      //  computation of time derivatives of the nodal values
      for (k=0;k<n;k++){
	tdlhs[k]=(lhs[k]-d[k])/dt/alpha;
      }
      
      //  physically corrected solution
      solution_correction ();    
      //  approximation of nodal values into ontegration points
      approximation ();
      
      
      // nulleqother ();
      actual_previous_nodval ();

    }
    
    
    // *********************************************************
    //  end of transport part and beginning of mechanical part
    // *********************************************************
    
    if (Mp->time <= Tp->time){
      
      //  approximation of temperatures from TRFEL nodes to MEFEL integration points
      approximation_temper ();
      //  approximation of humidities from TRFEL nodes to MEFEL integration points
      approximation_humid ();
      
      //  update of values stored at integration points
      Mm->updateipval();
      
      //  assembling of the right hand side (prescribed load)
      mefel_right_hand_side (lcid,f);

      //  computation of sums of force components in particular directions
      //  it is used in special creep and consolidation models
      Mb->comp_sum (f);
      
      //  computation of unbalanced forces which will be on the right hand side
      incr_internal_forces (lcid,fi);
      
      for (j=0;j<nm;j++){
	fb[j]=f[j]-fp[j]+fi[j];
	fp[j]=f[j];
      }    

      //  assembling of tangent stiffness matrix
      stiffness_matrix (lcid);
      
      //  solution of K(r).v=F
      Psolm->par_linear_solver (Gtm,Smat,dr,fb,Out,Mespr);

      //  new displacement vector
      for (j=0;j<nm;j++){
	r[j]+=dr[j];
      }
      
      //  computation of internal forces
      internal_forces (lcid,fi);
      
      //  temperature contributions
      nullv (fb,nm);
      Mb->dlc[lcid].tempercontrib (lcid,fb,nm,Cp->time);
      
      norfa=ss(f,f,nm);
      for (j=0;j<nm;j++){
	fb[j] = f[j] - fi[j] - fb[j];
	//fb[j] = f[j] - fi[j];
      }

      //  norm of vector of unbalanced forces
      norf=ss(fb,fb,nm);
      if (norfa > 0.0)
	norf /= norfa;
      
      
      if (norf<err){

      
	// computation of values required for output    
	compute_req_val (lcid);
	//  time increment
	Mp->time = Mp->timecon.newtime ();
	Mp->timecon.take_values (Mp->timecon);
	
	if (Mp->timecon.isitimptime ()==1){
	  mtsolver_save (aux,r,fp,ii,Mp->time,nm,0,0);
	}
	
	print_step(lcid, ii, Mp->time, f);
	print_flush();
	ii++;
      }
      
      // iteration of unbalanced forces caused by time independent models
      //  internal iteration loop
      else{
	for (j=0;j<ini;j++)
	  {
	    
	    if(Cp->fcsolv == fullnewtonc){
	      //cleaning of stiffness matrix
	      if (Smat != NULL){
		delete Smat;
		Smat=NULL;
	      }
	      //assembling of tangent stiffness matrix for fullnewton
	      stiffness_matrix (lcid);
	    }
	    
	    //  solution of K(r).v=F
	    Mp->ssle->solve_system (Gtm,Smat,dr,fb,Out);
	    
	    for (k=0;k<nm;k++)
	      r[k]+=dr[k];
	    
	    //  computation of internal forces
	    internal_forces (lcid,fi);
	    
	    //  vector of unbalanced forces
	    for (k=0;k<nm;k++)
	      fb[k]=f[k]-fi[k];
	    
	    //  norm of vector of unbalanced forces
	    norf=ss(fb,fb,nm);
	    if (norfa > 0.0)
	      norf /= norfa;
	    
	    if (Mespr==1) fprintf (stdout,"\n Time increment %lf   inner loop j %ld     norf=%e",Mp->time,j,norf);
	    if (norf<err)  break;
	  }
	if (j==ini || norf>err)
	  {
	    if (Mespr==1) fprintf (stdout,"\n Iteration process does not converge req. err = %e, reached err = %e", err,norf);
	    
	    // equilibrium state was not reached
	    compute_req_val (lcid);
	    //  time increment
	    Mp->time = Mp->timecon.newtime ();
	    Mp->timecon.take_values (Mp->timecon);
	    
	    if (Mp->timecon.isitimptime ()==1)
	      mtsolver_save (aux,r,fp,ii,Mp->time,nm,0,0);
	    print_step(lcid, ii, Mp->time, f);
	    print_flush();
	    ii++;
	  }
	else
	  {
	    // equilibrium state was reached
	    compute_req_val (lcid);
	    //  time increment
	    Mp->time = Mp->timecon.newtime ();
	    Mp->timecon.take_values (Mp->timecon);
	    
	    if (Mp->timecon.isitimptime ()==1)
	      mtsolver_save (aux,r,fp,ii,Mp->time,nm,0,0);
	    print_step(lcid, ii, Mp->time, f);
	    print_flush();
	    ii++;
	  }
      }
    }
    
    
    //  new time and new time increment
    Cp->time = Cp->timecon.newtime ();
    
    
    //Mp->time = Cp->time/86400.0;
    //Mp->timecon.take_values (Cp->timecon);
    //Mp->timecon.seconds_days ();
    
    
    Mp->time = Cp->time;
    Tp->time = Cp->time;
    
    
    dt = Cp->timecon.actualbacktimeincr ();
    i++;
    
    
  }while(Cp->time<end_time);
  
  print_close ();
  print_closet ();
  
  delete [] fi;
  delete [] fb;
  delete [] fp;
  delete [] dr;

  delete [] p;
  delete [] d;

  delete [] fit;
  delete [] fbt;
  delete [] lhst_last;
*/
}

