#include "mpi.h"
#include "npglvec.h"
#include "backupsolt.h"
#include "transprint.h"
#include "pnpsolvert.h"
#include "pglobalt.h"
#include "seqfilest.h"
#include <string.h>
#include <math.h>

/**
   function solves parallel nonstationary transport problem
   
   function selects a suitable numerical scheme for solution
   
   TKr, 19/02/2013
*/
void par_solve_nonstationary_problem ()
{
  if (Tp->tprob == discont_nonstat_problem){
    //  formulation based on nodal unknowns is used
    //par_linear_nonstat_solv_dform ();//not implemented
  }
  else{
    //  formulation based on unknown first nodal derivatives
    //  with respect to time is used
    par_nonstat_solv_vform_comp ();
    //par_linear_nonstat_solv_vform ();
    //par_linear_nonstat_solv_dform ();//not implemented
    //par_linear_nonstat_radiation_solv_dform ();//not implemented
  }
}


/**
   function solves parallel linear nonstationary transport problem
   the v-form version of the generalized trapezoidal method
   see T.J.R. Hughes: The Finite Element Method; Linear Static
   and Dynamic Finite Element Analysis
   
   according to JK, 20.12.2002 and TKo mtsolver.cpp
   revised 05/11/2012 by TKr
*/
void par_nonstat_solv_vform_comp ()
{
  long i,k,li,nsts,ret,n;
  double newtime,dt,dtmin,dtmax,dtdef,end_time;
  double *lhsb,*tdlhsb;
  np_glob_vec np_gv;
  long lcid = 0;
  
  //
  //  initialization phase
  //
  par_nonstat_solver_init(lcid, np_gv);
  
  //  initial time increment
  dt = Tp->timecont.initialtimeincr ();
  //  minimum time increment
  dtmin=Tp->timecont.dtmin;
  //  maximum time increment
  dtmax=Tp->timecont.dtmax;
  //  end time
  end_time = Tp->timecont.endtime ();
  
  //  number of step
  li = i = np_gv.istep;
  //  number of successful time steps (number of steps without inner iterations)
  nsts=0;
  // return status from the one_step function
  ret = 0;
  
  // only for adaptive time increment
  if(Tp->timecont.tct > 0){
    n=Ndoft;
    lhsb   = new double [n];
    tdlhsb   = new double [n];
    nullv (lhsb,n);
    nullv (tdlhsb,n);
  }

  // ***************************
  //  main iteration loop  ****
  // ***************************
  do{
    i++;
    
    newtime = Tp->time = Tp->timecont.newtime();
    //  computation of backward time step - only for adaptive time controler or nonconstant time increment
    if((Tp->timecont.timefun.tfunc != constant) && Tp->timecont.tct == 0)
      dt = Tp->timecont.actualbacktimeincr ();
    
    if(Tp->tprob == nonstationary_problem)
      // linear algorithm
      // perform one time step with linear solver
      ret = par_one_step_linear(lcid, newtime, dt, i, li, np_gv);
    
    
    if(Tp->tprob == nonlinear_nonstationary_problem){
      // non-linear algorithm
      // perform one time step with non-linear solver
      ret = par_one_step_nonlinear(lcid, newtime, dt, i, li, np_gv);
    }
    
    // handling with time controler for nonlinear solver
    if(Tp->tprob == nonlinear_nonstationary_problem && Tp->timecont.tct > 0){//only for adaptive time controler
      if (ret >= 0){ // equilibrium was attained
	
	for (k=0;k<n;k++){
	  lhsb[k] = np_gv.lhs[k]; //storing results from previous inner step
	  tdlhsb[k] = np_gv.tdlhs[k]; //storing results from previous inner step
	}
	
	if (ret == 0)
	  nsts++;      
	dtdef = Tp->timecont.actualforwtimeincr();
	if (nsts==2)
	  {
	    dt*=2.0;
	    nsts=0;
	    if (Myrank==0 && Mesprt==1)  
	      fprintf (stdout,"\n\n time increment is enlarged because no inner loop was neccessary in previous 3 steps");
	  }
	if (dt<=dtdef)
	  dt = dtdef;
	
	if (dt>dtmax)//maximum time increment
	  dt = dtmax;
      }//  end of the if (ret >= 0)
      else{
	//  reduction of the time increment because
	//  inner loop was not able to enforce equilibrium
	dt/=2.0;
	Tp->timecont.oldtime ();
	i--;
	
	for (k=0;k<n;k++){
	  np_gv.lhs[k] = lhsb[k]; //restoring results from previous time step
	  np_gv.tdlhs[k] = tdlhsb[k]; //restoring results from previous time step
	}
	
	if (Myrank==0 && Mesprt==1)  
	  fprintf (stdout,"\n\n time increment is reduced because the inner loop was not able to enforce equilibrium");
	
	if (dt<dtmin)
	  {
	    if (Myrank==0 && Mesprt==1)  fprintf (stderr,"\n\n time increment is less than minimum time increment");
	    if (Myrank==0 && Mesprt==1)  fprintf (stderr,"\n computation failed (file %s, line %d)\n",__FILE__,__LINE__);
	    if (Myrank==0 && Mesprt==1)  fprintf (stderr,"\n FORCED output of results from this step is performed\n");
	    compute_req_valt (lcid);
	    print_stept(lcid, i, Tp->time, np_gv.rhs);
	    print_flusht();
	    break;
	  }
      }//  end of else
    }//  if(Tp->tprob == nonlinear_nonstationary_problem && Tp->timecont.tct > 0)
      
  }while(Tp->time < end_time);
  
  print_closet();
  
  //Psolt->computation_stat_print (Outt);
  
  if(Tp->timecont.tct > 0){
    delete [] lhsb;
    delete [] tdlhsb;
  }
}



/**
  Function allocates and initializes global vectors used in npsolver
  It is used in one_step concept solver.

  Created by tomas Krejci according to Tomas Koudelka, 02/2013
*/

void par_nonstat_solver_init (long lcid, np_glob_vec &np_gv)
{
  long n;
  double dt;

  //  number of unknowns
  n=Ndoft;
  
  np_gv.alloc(n);
  //  nodal values
  np_gv.lhs = Lsrst->give_lhs (lcid);
  //  time derivatives of nodal values
  np_gv.tdlhs = Lsrst->give_tdlhs (lcid);
  //  prescribed fluxes / right hand side
  np_gv.rhs = Lsrst->give_rhs (lcid);
  
  if(Tp->tprob == nonlinear_nonstationary_problem)
    np_gv.alloc_aux(n);//auxiliary vectors

  //  initial time
  Tp->time=Tp->timecont.starttime ();
  //  initial time increment
  dt = Tp->timecont.initialtimeincr ();

  //  nodes - integration points interpolation
  approximation();  
  
  // pridal Madera 18.1.2008
  if (Tp->nvs==1 && Tp->pnvs==1)
    actual_previous_nodval ();
  
  //  inital printing of output and graphical informations
  if (Tp->hdbcont.restore_stat()){
    solvert_restore (np_gv.lhs, np_gv.tdlhs, np_gv.f, np_gv.istep, Tp->time, dt, Tp->timecont, n);
    //  initiation of transport material models
    Tm->initmaterialmodels();
    compute_req_valt (lcid);
    print_initt(-1, "at",Ptp->fni,Ptp->fei);
  }
  else{
    //  initiation of transport material models
    Tm->initmaterialmodels();
    compute_req_valt (lcid);
    print_initt(-1, "wt",Ptp->fni,Ptp->fei);
    print_stept(lcid, np_gv.istep, Tp->time, np_gv.rhs);
  }
  print_flusht();
}

/**
   function solves parallel linear nonstationary transport problem
   the v-form version of the generalized trapezoidal method
   see T.J.R. Hughes: The Finite Element Method; Linear Static
   and Dynamic Finite Element Analysis
   
   according to JK, 20.12.2002,
   revised 19/02/2013 by TKr
*/
long par_one_step_linear (long lcid,double time, double dt, long istep, long li, np_glob_vec &np_gv)
{
  long j,n;
  double alpha,*f,*d,*p,*lhs,*tdlhs,*rhs;
  
  //new time increment
  Tp->time = time;
  //  new step number
  Tp->istep = istep;
  
  //  nodal values
  lhs = np_gv.lhs;
  //  time derivatives of nodal values
  tdlhs = np_gv.tdlhs;
  //  prescribed fluxes / right hand side
  rhs = np_gv.rhs;
  //  vector of prescribed fluxes (right hand side)
  f = np_gv.f;
  //  predictor
  d = np_gv.d;
  //  auxiliary vector
  p = np_gv.p;
  
  //  coefficient in generalized trapezoidal integration rule
  alpha=Tp->alpha;

  //  number of transport degrees of freedom
  n=Ndoft;

  if (Myrank==0 && Mesprt==1)  fprintf (stdout,"\n\n -------------------------------------------------------------------");
  if (Myrank==0 && Mesprt==1)  fprintf (stdout,"\n PTRFEL Time step = %ld,  Time %e,  Time increment = %e",istep,Tp->time,dt);
  if (Myrank==0 && Mesprt==1)  fprintf (stdout,"\n -------------------------------------------------------------------\n");
  
  // update material properties and auxiliary values
  Tm->updateipval ();
  
  //  capacity matrix    
  capacity_matrix (lcid);
  
  //  conductivity matrix
  conductivity_matrix (lcid);
  
  if (istep == 0){
    Psolt->computation_stat (Gtt,Kmat);
  }

  //  predictor
  //  dd = d_n + (1-alpha) dt v_n
  for (j=0;j<n;j++){
    d[j]=lhs[j] + (1.0-alpha)*dt*tdlhs[j];
  }
  
  //  auxiliary vector
  //  K dd
  Kmat->gmxv (d,p);
  
  //  matrix of the system of equations
  //  C + K alpha dt
  Kmat->scalgm (dt*alpha);
  Kmat->addgm (1.0,*Cmat);
  
  //  prescribed nodal fluxes f_{n+1}
  trfel_right_hand_side (0,rhs,n);
  
  //  computation of the right hand side vector
  //  f_{n+1} - K dd
  for (j=0;j<n;j++){
    f[j] = rhs[j] - p[j];
  }
  
  //  solution of the system of algebraic equations
  //  (C + alpha dt K) v_{n+1} = f_{n+1} - K dd
  //  time derivatives v_{n+1} are solved
  Psolt->par_linear_solver (Gtt,Kmat,tdlhs,f,Outt,Mesprt);
  
  //  nodal values computed from nodal derivatives
  //  d_{n+1} = dd + alpha dt v_{n+1}
  for (j=0;j<n;j++){
    lhs[j] = d[j] + alpha*dt*tdlhs[j];
  }
    
  solution_correction ();
  //  nodes - integration points interpolation
  approximation ();
  
  // pridal Madera 18.1.2008
  if (Tp->nvs==1 && Tp->pnvs==1)
    actual_previous_nodval ();

  compute_req_valt (0);  
  print_stept(0,istep,Tp->time,rhs);
  print_flusht();
  
  if ((Tp->timecont.isitimptime ()==1) && Tp->hdbcont.save_stat())
    solvert_save (lhs,tdlhs,f,istep,Tp->time,dt,Tp->timecont,n);
  
  if (j == 0)
    return 0;
  
  return 1;
}

/**
   function solves parallel non-linear nonstationary transport problem  by Newton-Raphson method
   the v-form version of the generalized trapezoidal method
   see T.J.R. Hughes: The Finite Element Method; Linear Static
   and Dynamic Finite Element Analysis
   
   according to TKo
   revised 05/11/2012 by TKr
*/
long par_one_step_nonlinear (long lcid,double time, double dt, long istep, long li, np_glob_vec &np_gv)
{
  long j,k,n,ini;
  double alpha,*f,*d,*p,*lhs,*tdlhs,*rhs;
  double *fb,*fi;
  double norf_last;
  double zero,norf,*err,*thresh;  
  
  //new time increment
  Tp->time = time;
  //  new step number
  Tp->istep = istep;
  
  //  number of transport degrees of freedom
  n=Ndoft;

  //  nodal values
  lhs = np_gv.lhs;
  //  time derivatives of nodal values
  tdlhs = np_gv.tdlhs;
  //  prescribed fluxes / right hand side
  rhs = np_gv.rhs;
  //  vector of prescribed fluxes (right hand side)
  f = np_gv.f;
  //  predictor
  d = np_gv.d;
  //  auxiliary vector
  p = np_gv.p;

  // auxiliary vectors:
  fb = np_gv.fb;
  fi = np_gv.fi;
  
  //  initial values
  nullv (fb,n);
  nullv (fi,n);

  
  //  maximum number of iterations in inner loop
  ini = Tp->nii;
  //  required norm of vector of unbalanced forces
  err = Tp->errarr;
  //  threshold for the size of the right hand side
  thresh=Tp->threshrhs;

  //  coefficient in generalized trapezoidal integration rule
  alpha=Tp->alpha;
  //  computer zero
  zero=Tp->zero;

  if (Myrank==0 && Mesprt==1)  fprintf (stdout,"\n\n -------------------------------------------------------------------");
  if (Myrank==0 && Mesprt==1)  fprintf (stdout,"\n PTRFEL Time step = %ld,  Time %e,  Time increment = %e",istep,Tp->time,dt);
  if (Myrank==0 && Mesprt==1)  fprintf (stdout,"\n -------------------------------------------------------------------\n");
  
  //  number of transport degrees of freedom
  n=Ndoft;
  
  // update material properties and auxiliary values
  Tm->updateipval ();
  
  //  capacity matrix    
  capacity_matrix (lcid);
  
  //  conductivity matrix
  conductivity_matrix (lcid);

  if (istep == 0){
    Psolt->computation_stat (Gtt,Kmat);
  }
  
  //  predictor
  //  dd = d_n + (1-alpha) dt v_n
  for (j=0;j<n;j++){
    d[j]=lhs[j] + (1.0-alpha)*dt*tdlhs[j];
  }
  
  //  auxiliary vector
  //  K dd
  Kmat->gmxv (d,p);
  
  //  matrix of the system of equations
  //  C + K alpha dt
  Kmat->scalgm (dt*alpha);
  Kmat->addgm (1.0,*Cmat);
  Kmat->copygm (*Cmat);

  //  prescribed nodal fluxes f_{n+1}
  trfel_right_hand_side (0,rhs,n);
  
  //  computation of the right hand side vector
  //  f_{n+1} - K dd
  for (j=0;j<n;j++){
    f[j] = rhs[j] - p[j];
  }
  
  //  solution of the system of algebraic equations
  //  (C + alpha dt K) v_{n+1} = f_{n+1} - K dd
  //  time derivatives v_{n+1} are solved
  Psolt->par_linear_solver (Gtt,Kmat,tdlhs,f,Outt,Mesprt);
  
  //  nodal values computed from nodal derivatives
  //  d_{n+1} = dd + alpha dt v_{n+1}
  for (j=0;j<n;j++){
    lhs[j] = d[j] + alpha*dt*tdlhs[j];
  }

  //  physically corrected solution
  solution_correction ();
  //  nodes - integration points interpolation
  approximation ();
  
  // pridal Madera 18.1.2008
  if (Tp->nvs==1 && Tp->pnvs==1)
    actual_previous_nodval ();
  
  //////////////////////////////////////////////////
  norf_last = 1.0e20;
  //inner iteration loop//////////////////
  for (j=0;j<ini;j++){
    
    // full newton-raphson
    if (Tp->trsolv == fullnewtont){
      // matrices are computing in each inner iteration
      // capacity matrix
      capacity_matrix (0);
      
      //  conductivity matrix
      conductivity_matrix (0);
      
      //  auxiliary vector  K (d+(1-alpha)*dt*v)
      Kmat->gmxv (d,p);
      
      //  matrix of the system of equations
      //  C + alpha.dt.K
      Kmat->scalgm (dt*alpha);
      Kmat->addgm (1.0,*Cmat);
      Kmat->copygm (*Cmat);
    }
    
    if (Tp->trestype==lrhst){
      //  correct right hand side of transport part
      trfel_right_hand_side (lcid,rhs,n);
      // Solver computes residuum from system of equations
      Kmat->gmxv (tdlhs,fi);//new fi vector
      // Res. = F - K (d+(1-alpha)*dt*v) - (C + alpha*dt*K)*v 
      for (k=0;k<n;k++){
	fb[k] = rhs[k] - p[k] - fi[k];
      }
    }
    
    if (Tp->trestype==fluxest){
      // Solver computes unbalanced fluxes
      internal_fluxes (fi,n);//new fi vector
      //  vector of unbalanced fluxes
      for (k=0;k<n;k++){
	fb[k]=fi[k];
      }
    }
    
    norf = Psolt->unbalanced_values (fb,Outt);//norf = ss (fb,fb,n);

    if (Myrank==0 && Mesprt==1)  fprintf (stdout,"\n inner iteration %ld   error %e",j,norf);
    if (norf<err[0]) break;
    
    Psolt->par_linear_solver (Gtt,Cmat,fi,fb,Outt,Mesprt);//fi is output now
    
    for (k=0;k<n;k++){
      tdlhs[k]+=fi[k];
      lhs[k]+=alpha*dt*fi[k];
    }
    
    //  physically corrected solution
    solution_correction ();
    //  approximation of nodal values into ontegration points
    approximation ();
    
    // vlozil JM 15.1.2008
    // nulleqother ();
    if (Tp->nvs==1 && Tp->pnvs==1)
      actual_previous_nodval ();
    
    //  convergence control for newton-raphson
    //  condition of decreesing of error
    if (Tp->convergcontrolt==yes){
      if (norf > norf_last){
		
	if (Myrank==0 && Mesprt==1)  
	  fprintf (stdout,"\n\n convergence control: inner iteration skiped %ld error %e\n\n",j,norf);	  
	
	return -2;//break;
      }
            
      norf_last = norf; //storing norf from previous inner step
    }
  }  
  
  if(norf >= err[0])
    return -1;

  //////////////////////////////////////////////////
  //  printing of output and graphical informations
  compute_req_valt (lcid);
  print_stept(lcid,istep,Tp->time,rhs);//print_stept(0,i,Tp->time,NULL);
  print_flusht();
  
  if ((Tp->timecont.isitimptime ()==1) && Tp->hdbcont.save_stat())
    solvert_save (lhs,tdlhs,f,istep,Tp->time,dt,Tp->timecont,n);
  
  return 0;
}


/**
   function solves linear nonstationary transport problem
   the v-form version of the generalized trapezoidal method
   see T.J.R. Hughes: The Finite Element Method; Linear Static
   and Dynamic Finite Element Analysis
   
   JK
*/
void par_linear_nonstat_solv_vform ()
{
  long i,j,n;
  double s,zero,alpha,*d,*p,*lhs,*lhsi,*tdlhs,*rhs;

  double dt,end_time;
  

  n=Ndoft;
  
  lhs = Lsrst->give_lhs (0);
  tdlhs = Lsrst->give_tdlhs (0);
  lhsi = Lsrst->lhsi;
  rhs = Lsrst->give_rhs (0);

  d = new double [n];
  p = new double [n];
  //tdlhs = new double [n];

  //  initial values
  nullv (lhs,n);
  nullv (tdlhs,n);
  nullv (d,n);
  nullv (p,n);
  


  
  alpha=Tp->alpha;

  zero=Tp->zero;
  



  // **************************
  //  main iteration loop  ***
  // **************************

  //  loop counter
  i=0;
  
  //  starting time
  Tp->time = Tp->timecont.starttime ();
  //  time increment
  dt = Tp->timecont.initialtimeincr ();
  //  end time
  end_time = Tp->timecont.endtime ();
  

  //  nodes - integration points interpolation
  approximation ();
  actual_previous_change ();

  Tm->initmaterialmodels();
  

  print_initt(-1, "wt",Ptp->fni,Ptp->fei);
  print_stept(0,i,Tp->time,NULL);
  print_flusht();


  do{
    

    if (Myrank==0) fprintf (stdout,"\n iteration number %ld",i);
    
    //fprintf (Outt,"\n\n\n\n\n iteration number %ld",i);
    
    //par_aux_nonstat_print (gr,lhsi,d,time);
    
    
    //fprintf (Out,"\n\n\n\n\n\n kontrola integracnich bodu");
    //for (long ijk=0;ijk<Tm->nip;ijk++){
    //fprintf (Out,"\n %ld  %lf %lf",ijk,Tm->ip[ijk].av[0],Tm->ip[ijk].av[1]);
    //}

    
    
    conductivity_matrix (0);

    capacity_matrix (0);
    
    if (i==0){
      Psolt->computation_stat (Gtt,Kmat);
    }
    
    //Kmat->printdiag (Outt);
    //fprintf (Outt,"\n\n MATICE K \n");
    //Kmat->printmat (Outt);
    //fprintf (Outt,"\n\n MATICE C \n");
    //Cmat->printmat (Outt);

    for (j=0;j<n;j++){
      p[j]=lhs[j]+(1.0-alpha)*dt*tdlhs[j];
    }
    
    //  auxiliary vector
    Kmat->gmxv (p,d);
    
    

    //fprintf (Outt,"\n\n\n\n\n kontrola K.(d+dt*(1-a)*v)");
    //for (long ijk=0;ijk<n;ijk++){
    //fprintf (Outt,"\n %ld %le",ijk,d[ijk]);
    //}
    

    
    Kmat->scalgm (alpha*dt);
    Kmat->addgm (1.0,*Cmat);
    
    
    
    //fprintf (Out,"\n\n\n\n\n kontrola d+dt*(1-a)*v");
    //for (long ijk=0;ijk<n;ijk++){
    //fprintf (Out,"\n %ld %lf",ijk,p[ijk]);
    //}

    
    
    
    //fprintf (Out,"\n\n\n\n\n kontrola K.(d+dt*(1-a)*v)");
    //for (long ijk=0;ijk<n;ijk++){
    //fprintf (Out,"\n %ld %lf",ijk,r[ijk]);
    //}

    
    //  right hand side
    trfel_right_hand_side (0,rhs,n);
    
    //fprintf (Outt,"\n\n\n\n\n kontrola prave strany");
    for (long ijk=0;ijk<n;ijk++){
      fprintf (Outt,"\n rhs %ld %le",ijk,rhs[ijk]);
    }
    
    
    
    for (j=0;j<n;j++){
      rhs[j] = rhs[j] - d[j];
      d[j]=tdlhs[j];
    }
    
    
    
    //fprintf (Outt,"\n\n\n\n\n kontrola prave strany");
    for (long ijk=0;ijk<n;ijk++){
      fprintf (Outt,"\n rhs2 %ld %le",ijk,rhs[ijk]);
    }
    
 
    //fprintf (Outt,"\n\n MATICE SOUSTAVY \n");
    Kmat->printmat (Outt);




    //par_linear_solver ();
    Psolt->par_linear_solver (Gtt,Kmat,tdlhs,rhs,Outt,Mesprt);
    



    //fprintf (Outt,"\n\n MATICE SOUSTAVY PO RESENI \n");
    //Kmat->printmat (Outt);
    
    //fprintf (Outt,"\n\n\n\n\n kontrola reseni soustavy");
    //for (long ijk=0;ijk<n;ijk++){
    //fprintf (Outt,"\n tdlhs %ld      %le",ijk,tdlhs[ijk]);
    //}
    
    
    
    for (j=0;j<n;j++){
      s=(1.0-alpha)*d[j]+alpha*tdlhs[j];
      lhs[j]+=dt*s;
    }
    
    
    //fprintf (Outt,"\n\n\n\n\n kontrola obnovenych promennych");
    for (long ijk=0;ijk<n;ijk++){
      fprintf (Outt,"\n lhs %ld   %le",ijk,lhs[ijk]);
    }

    //  nodes - integration points interpolation
    approximation ();
    


    Tp->time = Tp->timecont.newtime ();
    dt = Tp->timecont.actualbacktimeincr ();
    i++;
    
    print_stept(0,i,Tp->time,NULL);
    print_flusht();

    printf ("\n Tp->time   %e",Tp->time);
    printf ("\n i          %ld",i);
    
  }while(Tp->time<end_time);



  delete [] p;  delete [] d;
  //fclose (gr); 

  print_closet();
  
  Psolt->computation_stat_print (Outt);
  
  
}

