/*
    File:            consol_awf1.cpp
    Author:          Tomas Krejci, 12/09/2008, revised 13/04/2018
    Purpose:         material model for saturated-nonsaturated one-phase flow (water) in deforming medium
    sources:         Lewis and Schrefler's book pp. 86-89, initial material parameters are set for benchmark on page n. 168;
    another sources: geomkp.pdf from michal Sejnoha;
		     
		     P. Kuklik, J. Mares and M. Sejnoha (1999) The structural strenght of soil
		     from the isotropic consolidation point of view. Computational Mechanics for the Next
		     Millenium. Wang, Lee, Ang (ed.), Vol. 2, Elsevier, Singpore, 797-802;

		     P. Kuklik (2010) Preconsolidation, Structural Strength of soil, and its effect on subsoil 
		     upper structure intraction; Professor thesis

    unknowns:        number of unknowns=1, pw = liqud water pressure
*/ 

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "aliast.h"
#include "constrel.h"
#include "consol_awf1.h"
#include "globalt.h"
#include "globmatt.h"

con_awf1mat::con_awf1mat()
{
  compress = 0; //compressible grains: 0=no; 1=yes

  alpha = 1.0;//Biot's constant [-] alpha = 1 - kt/ks
  ks = 2.167e9;//bulk modulus of solid phase (grains) [Pa]
  kw = 2.0e9;//bulk modulus of water [Pa]
  phi0 = 0.297;//initial porosity [-]
  kintr = 4.5e-13;//intrinsic permeability [m^2]
  rhow = 1000.0;//water density [kg/m^3]
  muw0 = 1.0e-3;//water viscosity [Pa.s]
  /////////////////////////////////////////////////////////
  k0 = 32e-11;//
  e0 = 0.52;//
  lam0 = 0.001;//
  lam = 0.03;//
  m = 9;//
  sigma_m_bar = 100000.0;//initial effective mean stress
  /////////////////////////////////////////////////////////
  kappa = 0.012;//
  sigma_m_eff_bar = 0.0;//initial (actual) effective mean stress = preconsolidation pressure
  sigma_m_tot_bar = 0.0;//initial (actual) total mean stress
  sigma_m_eff_init = 1.0e-3; //initial effective mean stress
  /////////////////////////////////////////////////////////

  mefel_units = 1.0;//basic units for pressures = Pa (Pascals)
  rhos0 = 2500.0; //solid density
  pw_bc = 0.0; //free boundary pressure
}

con_awf1mat::~con_awf1mat()
{}


/**
   function computes conductivity %matrix of the material
   in the required integration point
   
   @param d - conductivity %matrix of material
   @param ri  - row index
   @param ci  - column index   
   @param ipp - number of integration point

*/
void con_awf1mat::matcond (matrix &d,long ri,long ci,long ipp)
{
  long n;
  n = d.n;

  switch (n){
  case 1:{
    matcond1d (d,ri,ci,ipp);//1D
    break;
  }
  case 2:{
    matcond2d (d,ri,ci,ipp);//2D
    break;
  }
  case 3:{
    matcond3d (d,ri,ci,ipp);//3D
    break;
  }
  default:{
    print_err("unknown number of components of conductivity tensor is required",__FILE__,__LINE__,__func__);
  }
  }
}


/**
   function creates conductivity %matrix of the material for 1D problems

   @param d   - conductivity %matrix of the material
   @param ri  - row index
   @param ci  - column index   
   @param ipp - number of integration point
*/
void con_awf1mat::matcond1d (matrix &d,long ri,long ci,long ipp)
{
  double kk;
  double pw;
  
  pw = Tm->ip[ipp].av[0];
  kk = get_kww(pw,ipp);
  
  fillm(0.0,d);

  d[0][0] = kk;
}

/**
   function creates conductivity %matrix of the material for 2D problems

   @param d   - conductivity %matrix of the material
   @param ri  - row index
   @param ci  - column index   
   @param ipp - number of integration point
*/
void con_awf1mat::matcond2d (matrix &d,long ri,long ci,long ipp)
{
  double kk;
  double pw;
  
  pw = Tm->ip[ipp].av[0];
  kk = get_kww(pw,ipp);
  
  fillm(0.0,d);
  
  d[0][0] = kk;   d[0][1] = 0.0;
  d[1][0] = 0.0;  d[1][1] = kk;
}

/**
   function creates conductivity %matrix of the material for 3D problems

   @param d   - conductivity %matrix of the material
   @param ri  - row index
   @param ci  - column index   
   @param ipp - number of integration point
*/

void con_awf1mat::matcond3d (matrix &d,long ri,long ci,long ipp)
{
  double kk;
  double pw;
  
  pw = Tm->ip[ipp].av[0];
  kk = get_kww(pw,ipp);
  
  fillm(0.0,d);
  
  d[0][0]=kk;   d[0][1]=0.0;  d[0][2]=0.0;
  d[1][0]=0.0;  d[1][1]=kk;   d[1][2]=0.0;
  d[2][0]=0.0;  d[2][1]=0.0;  d[2][2]=kk;
}


/**
   function creates capacity %matrix of the material

   @param c   - capacity %matrix of the material
   @param ri  - row index
   @param ci  - column index   
   @param ipp - number of integration point
*/
void con_awf1mat::matcap (double &cc,long ri,long ci,long ipp)
{
  double pw;
  
  pw = Tm->ip[ipp].av[0];
  cc = get_capww(pw,ipp);
}

/**
   function reads parameters
   
   @param in - input file

   12/9/2008, TKr
*/
void con_awf1mat::read(XFILE *in)
{
  
  xfscanf (in,"%k%m","waterflowtype",&waterflowtype_kwdset, &model_type);
  xfscanf (in,"%d", &compress);

  switch (model_type){
  case lewis_and_schrefler:{//Lewis and Schrefler's book
    xfscanf (in,"%le %le %le %le %le %le %le %le", &alpha, &ks, &phi0, &kw, &rhow, &muw0, &kintr, &rhos0);
    lewis_ret.read(in);
    break;
  }
  case lewis_and_schrefler_mefel:{//Lewis and Schrefler's book
    xfscanf (in,"%le %le %le %le %le %le %le %le %le %le", &mefel_units, &alpha, &ks, &phi0, &kw, &rhow, &muw0, &kintr, &rhos0, &pw_bc);
    break;
  }
  case lewis_and_schrefler_mefel_table:{//Lewis and Schrefler's book
    xfscanf (in,"%le %le %le %le %le %le %le %le %le %le", &mefel_units, &alpha, &ks, &phi0, &kw, &rhow, &muw0, &kintr, &rhos0, &pw_bc);

    //reading of retention curve:
    data.read (in);
    
    break;
  }
  case gardner_exponential:{//partially saturated medium = Exponential model, Gardner 
    gardner_ret.read(in);
    break;
  }
  case potts_log_linear:{//partially saturated medium = Log linear model, Potts
    potts_ret.read(in);
    break;
  }
  case van_genuchten:{//partially saturated medium =Van Genuchten model
    xfscanf (in,"%le %le %le %le %le %le %le %le %le %le", &mefel_units, &alpha, &ks, &phi0, &kw, &rhow, &muw0, &kintr, &rhos0, &pw_bc);
    van_genuchten_ret.read(in);
    break;
  }
  case kuklik_camclay:{//fully saturated isotropic consolidation according to Kuklik, with changing porosity from cam-clay model
    xfscanf (in,"%le %le %le %le %le %le %le %le", &k0, &e0, &kappa, &lam, &m, &gamaw, &sigma_m_eff_bar, &sigma_m_tot_bar);
    break;
  }
  case kuklik_camclay_mech:{//fully saturated isotropic consolidation according to Kuklik, with changing porosity from MEFEL cam-clay model
    xfscanf (in,"%le %le %le %le %le %le %le", &k0, &e0, &kappa, &lam, &m, &gamaw, &sigma_m_eff_init);
    break;
  }
  default:{
    print_err ("unknown model type is required", __FILE__, __LINE__, __func__);
    abort();
  }
  } 
}


/**
   function reads parameters
   
   @param in - input file

   01/05/2008, TKr
*/
void con_awf1mat::print(FILE *out)
{
  
  fprintf (out,"\n %d ", int(model_type));
  fprintf (out,"\n %d ", compress);
  
  switch (model_type){
  case lewis_and_schrefler:{//Lewis and Schrefler's book
    fprintf (out,"\n %le %le %le %le %le %le %le %le \n", alpha, ks, phi0, kw, rhow, muw0, kintr, rhos0);
    lewis_ret.print(out);
    break;
  }
  case lewis_and_schrefler_mefel:{//Lewis and Schrefler's book
    fprintf (out,"\n %le %le %le %le %le %le %le %le %le %le \n", mefel_units, alpha, ks, phi0, kw, rhow, muw0, kintr, rhos0, pw_bc);
    break;
  }

  case lewis_and_schrefler_mefel_table:{//Lewis and Schrefler's book
    fprintf (out,"\n %le %le %le %le %le %le %le %le %le %le \n", mefel_units, alpha, ks, phi0, kw, rhow, muw0, kintr, rhos0, pw_bc);

    //printing of retention curve:
    data.print (out);
    fprintf (out,"\n");

    break;
  }

  case gardner_exponential:{//partially saturated medium = Exponential model, Gardner 
    gardner_ret.print(out);
    break;
  }
  case potts_log_linear:{//partially saturated medium = Log linear model, Potts
    potts_ret.print(out);
    break;
  }
  case van_genuchten:{//partially saturated medium =Van Genuchten model
    fprintf (out,"\n %le %le %le %le %le %le %le %le %le %le \n", mefel_units, alpha, ks, phi0, kw, rhow, muw0, kintr, rhos0, pw_bc);
    van_genuchten_ret.print(out);
    break;
  }
  case kuklik_camclay:{//fully saturated isotropic consolidation according to Kuklik, with changing porosity from cam-clay model
    fprintf (out,"\n %le %le %le %le %le %le %le %le \n", k0, e0, kappa, lam, m, gamaw, sigma_m_eff_bar, sigma_m_tot_bar);
    break;
  }
  case kuklik_camclay_mech:{//fully saturated isotropic consolidation according to Kuklik, with changing porosity from MEFEL cam-clay model
    fprintf (out,"\n %le %le %le %le %le %le %le \n", k0, e0, kappa, lam, m, gamaw, sigma_m_eff_init);
    break;
  }
  default:{
    print_err ("unknown model type is required", __FILE__, __LINE__, __func__);
    abort();
  }
  } 
}





/**
   function computes volume part of right-hand side matrix
   in the required integration point
   
   @param d - right-hand side %matrix of material
   @param ri  - row index
   @param ci  - column index   
   @param ipp - number of integration point
*/
void con_awf1mat::rhs_volume (matrix &d,long ri,long ci,long ipp)
{
  long m;
  m = d.m;
  
  switch (m){
  case 1:{
    rhs1d1 (d,ri,ci,ipp);//1D
    break;
  }
  case 2:{
    rhs2d1 (d,ri,ci,ipp);//2D
    break;
  }
  case 3:{
    rhs3d1 (d,ri,ci,ipp);//3D
    break;
  }
  default:{
    print_err ("unknown number of components of stress tensor is required", __FILE__, __LINE__, __func__);
    abort();
  }
  }
}


/**
   function computes volume part 2 of right-hand side matrix
   in the required integration point
   
   @param cc - right-hand side coefficient of material
   @param ri  - row index
   @param ci  - column index   
   @param ipp - number of integration point
*/
void con_awf1mat::rhs_volume2 (double &cc,long ri,long ci,long ipp)
{
  double pw;
  
  pw = Tm->ip[ipp].av[0];
  cc = get_fwu(pw,ipp);
}



/**
   function creates volume right-hand side matrix of the material for 1D problems
   @param d - right-hand %matrix of the material
   @param ri  - row index
   @param ci  - column index   
   @param ipp - number of integration point
*/
void con_awf1mat::rhs1d1 (matrix &d,long ri,long ci,long ipp)
{
  double f,pw;
  
  pw = Tm->ip[ipp].av[0];
  f = get_fw1(pw,ipp);
  fillm(0.0,d);
  
  d[0][0] = f*Tp->gr[0];
}

/**
   function creates volume right-hand side matrix of the material for 2D problems
   @param d - right-hand %matrix of the material
   @param ri  - row index
   @param ci  - column index   
   @param ipp - number of integration point
*/
void con_awf1mat::rhs2d1 (matrix &d,long ri,long ci,long ipp)
{
  double f,pw;
  
  pw = Tm->ip[ipp].av[0];
  
  f = get_fw1(pw,ipp);
  fillm(0.0,d);

  d[0][0] = f*Tp->gr[0];
  d[1][0] = f*Tp->gr[1];
}

/**
   function creates volume right-hand side matrix of the material for 3D problems
   @param d - right-hand %matrix of the material
   @param ri  - row index
   @param ci  - column index   
   @param ipp - number of integration point
*/
void con_awf1mat::rhs3d1 (matrix &d,long ri,long ci,long ipp)
{
  double f,pw;
  
  pw = Tm->ip[ipp].av[0];
  
  f = get_fw1(pw,ipp);
  fillm(0.0,d);

  d[0][0] = f*Tp->gr[0];
  d[1][0] = f*Tp->gr[1];
  d[2][0] = f*Tp->gr[2];
}




/**
   function computes degree of saturation(sorption curve)
   @param pw - water pressure

   @retval sw - degree of saturation

   12/9/2008, TKr
*/
double con_awf1mat::get_sw(double pw, long ipp)
{
  double sw,pc;
  
  switch (model_type){
  case lewis_and_schrefler:{//Lewis and Schrefler's book
    sw = lewis_ret.sw(pw);
    break;
  }
  case lewis_and_schrefler_mefel:{//saturation degree ind its derivative are obtained from mefel;
    if(Tm->nontransq != NULL){
      sw = Tm->givenontransq(saturation_deg, ipp); //actual saturation degree
    }
    else{//zkontrolovat??!!
      //this below if for stability during tests
      pc = -pw;
      if(pc > 1)
	sw = 1.0 - 0.05125*log(pc);
      else
	sw = 1.0;
    }
    break;
  }

  case lewis_and_schrefler_mefel_table:{//saturation degree and its derivative are obtained from table;
    //actual saturation degree    
    if (data.tfunc == tab)
      {
	if ((pw < data.tabf->x[0]) || (pw > data.tabf->x[data.tabf->asize-1]))
	  {
	    print_err("required value %le is out of table range <%le;%le> on ip=%ld\n", 
		      __FILE__, __LINE__, __func__, pw, data.tabf->x[0], data.tabf->x[data.tabf->asize-1], ipp);
	    abort();
	  }
      }
    sw = data.getval (pw);
    
    break;
  }

  case gardner_exponential:{//partially saturated medium = Exponential model, Gardner 
    sw = gardner_ret.sw(pw);
    break;
  }
  case potts_log_linear:{//partially saturated medium = Log linear model, Potts
    sw = potts_ret.sw(pw);
    break;
  }
  case van_genuchten:{//partially saturated medium = Van Genuchten model
    sw = van_genuchten_ret.sw(pw);
    break;
  }
  case kuklik_camclay:{//isotropic consolidation according to Kuklik with changing porosity from cam-clay model
    sw = 1.0;
    break;
  }
  case kuklik_camclay_mech:{//isotropic consolidation according to Kuklik with changing porosity from MEFEL cam-clay model
    sw = 1.0;
    break;
  }
  default:{
    print_err ("unknown model type is required", __FILE__, __LINE__, __func__);
    abort();
  }
  } 
  return(sw);
}


/**
   function computes partial derivative of degree of saturation with respect to pw, specific wter content
   @param pw - water pressure

   @retval dsw_dpw - partial derivative of degree of saturation with respect to pw

   12/9/2008, TKr
*/
double con_awf1mat::get_dsw_dpw(double pw, long ipp)
{
  double dsw_dpw,pc;
  double sw;

  switch (model_type){
  case lewis_and_schrefler:{//Lewis and Schrefler's book
    dsw_dpw = lewis_ret.dsw_dpw(pw);
    break;
  }
  case lewis_and_schrefler_mefel:{//saturation degree ind its derivative are obtained from mefel;
    if(Tm->nontransq != NULL){
      sw = Tm->givenontransq(saturation_deg, ipp); //actual saturation degree
      dsw_dpw = Tm->givenontransq(der_saturation_deg, ipp); //actual derivative of saturation degree
      dsw_dpw = dsw_dpw/mefel_units; //basic units = Pa//debug??!!
    
      //correction of dsw_dpw
      //if(sw > 0.99)
      //if(dsw_dpw < 1.0e-8){
      //  dsw_dpw = 1.0e-8;
      //}
    }
    else{
      //this below is for testing:
      pc = -pw;
      if(pc > 1)
	dsw_dpw = 0.05125/pc;
      else
	dsw_dpw = 0.0;
    }
    break;
  }

  case lewis_and_schrefler_mefel_table:{//saturation degree and its derivative are obtained from table;
    //sw = Tm->givenontransq(saturation_deg, ipp); //actual saturation degree
    dsw_dpw = data.getderiv (pw); //actual derivative of saturation degree
    break;
  }

  case gardner_exponential:{//partially saturated medium = Exponential model, Gardner 
    dsw_dpw = gardner_ret.dsw_dpw(pw);
    break;
  }
  case potts_log_linear:{//partially saturated medium = Log linear model, Potts
    dsw_dpw = potts_ret.dsw_dpw(pw);
    break;
  }
  case van_genuchten:{//partially saturated medium = Van Genuchten model
    dsw_dpw = van_genuchten_ret.dsw_dpw(pw);
    break;
  }
  default:{
    print_err ("unknown model type is required", __FILE__, __LINE__, __func__);
    abort();
  }
  } 

  return(dsw_dpw);
}


/**
   function computes water relative permeability
   @param pw - water pressure
   
   @retval krw - water relative permeability

   12/9/2008, TKr
*/
double con_awf1mat::get_krw(double pw, long ipp)
{
  double krw,sw;
  
  switch (model_type){
  case lewis_and_schrefler:{//Lewis and Schrefler's book
    sw = get_sw(pw,ipp);
    krw = lewis_ret.krw(sw);
    //krw=1.0;
    break;
  }
  case lewis_and_schrefler_mefel_table:
  case lewis_and_schrefler_mefel:{//Lewis and Schrefler's book, saturation degree ind its derivative are obtained from mefel;
    krw=1.0;
    break;
  }
  case gardner_exponential:{//partially saturated medium = Exponential model, Gardner 
    krw = gardner_ret.krw(pw);
    break;
  }
  case potts_log_linear:{//partially saturated medium = Log linear model, Potts
    krw = potts_ret.krw(pw);
    break;
  }
  case van_genuchten:{//partially saturated medium = Van Genuchten model
    krw = van_genuchten_ret.krw(pw);
    break;
  }
  default:{
    print_err ("unknown model type is required", __FILE__, __LINE__, __func__);
    abort();
  }
  } 

  return(krw);
}


/**
   function returns porosity

   @retval phi - porosity

   12/9/2008, TKr
*/
double con_awf1mat::give_porosity(long ipp)
{
  double por;

  switch (model_type){
  case lewis_and_schrefler:{//Lewis and Schrefler's book
    por = phi0; //constant value
    break;
  }
  case lewis_and_schrefler_mefel_table:{
    por = phi0; //constant value
    break;
  }
  case lewis_and_schrefler_mefel:{//Lewis and Schrefler's book
    if(Tm->nontransq != NULL){
      por = Tm->givenontransq(porosity, ipp);// from mefel
    }
    else
      por = phi0;//for testing
    break;
  }
  case gardner_exponential:
  case potts_log_linear:
  case van_genuchten:{
    por = phi0; //constant value
    break;
  }
  default:{
    print_err ("unknown model type is required", __FILE__, __LINE__, __func__);
    abort();
  }
  } 

  return(por);
}


/**
   function computes intrinsic permeability

   @retval kintr - intrinsic permeability

   12/9/2008, TKr
*/
double con_awf1mat::get_kintr()
{
  return(kintr);
}


/**
   function computes water conductivity of porous material 

   @retval k - conductivity

   12/9/2008, TKr
*/
double con_awf1mat::get_k(double pw,long ipp)
{
  double k;

  switch (model_type){
  case gardner_exponential:{//partially saturated medium = Exponential model, Gardner 
    k = gardner_ret.get_k(pw);
    break;
  }
  case potts_log_linear:{//partially saturated medium = Log linear model, Potts
    k = potts_ret.get_k(pw);
    break;
  }
  case van_genuchten:{//partially saturated medium = Van Genuchten model
    k = van_genuchten_ret.get_k(pw);
    break;
  }
  default:{
    print_err ("unknown model type is required", __FILE__, __LINE__, __func__);
    abort();
  }
  } 
  return(k);
}


/**
   function returns Biot's constant

   @retval alpha - Biot's constant

   12/9/2008, TKr
*/
double con_awf1mat::get_alpha()
{
  return(alpha);
}


/**
   function returns kw bulk modulus of water [Pa]

   @retval kw - bulk modulus of water [Pa]

   12/9/2008, TKr
*/
double con_awf1mat::get_kw()
{
  return(kw);
}


/**
   function returns muw water viscosity [Pa.s]

   @retval muw - water viscosity [Pa.s]

   12/9/2008, TKr
*/
double con_awf1mat::get_muw()
{
  return(muw0);
}


/**
   function creates conductivity coefficient of the general material
   @param pw - water pressure

   @retval kww - conductivity coefficient

   12/9/2008, TKr
*/
double con_awf1mat::get_kww(double pw, long ipp)
{
  double s,ds;
  double krw,kww;
  double e,eps_v,sigma_m_eff,e0_bar;
  double a,b,c,e0_hat,lam_hat;
  
  
  switch (model_type){
  case lewis_and_schrefler:
  case van_genuchten:
  case lewis_and_schrefler_mefel_table:
  case lewis_and_schrefler_mefel:{//Lewis and Schrefler's book
    krw = get_krw(pw,ipp);
    kww = krw*kintr/muw0;//p. 89
    break;
  }
  case gardner_exponential:
  case potts_log_linear:
    //case van_genuchten:
    {
      double k;
      
      k   = get_k(pw,ipp);
      krw = get_krw(pw,ipp);
      kww = krw*k;    
      break;
    }
  case kuklik_camclay:{//isotropic consolidation according to Kuklik with changing porosity from cam-clay model
    
    sigma_m_eff = sigma_m_tot_bar + pw;

    /*
      if (sigma_m_eff_bar >= 0.0)
      e0_bar = e0;
      else
      e0_bar = (lam + kappa)*log(-sigma_m_eff_bar);
      
      if(sigma_m_eff > sigma_m_eff_bar){//swelling (unloading) line
      eps_v = -kappa/(1+e0_bar)*log(-sigma_m_eff);
      //e = e0_bar*(1 + eps_v) + eps_v;
      e = -kappa*log(-sigma_m_eff) + e0_bar;//another expression
      kww = -k0*(1+e0_bar)/kappa/gamaw*sigma_m_eff*pow((e/e0),m);
      }
      else{ //NCL
      eps_v = -lam/(1+e0)*log(-sigma_m_eff);
      //e = e0*(1 + eps_v) + eps_v;
      e = -lam*log(-sigma_m_eff) + e0;//another expression
      kww = -k0*(1+e0)/lam/gamaw*sigma_m_eff*pow((e/e0),m);
      }
    */

    e0_bar = (kappa - lam)*log(-sigma_m_eff_bar) + e0;
    c = e0_bar;
    b = -kappa;
    a = -b*(lam/e0) - c*(lam/e0)*(lam/e0);
    e0_hat = -2*a*(log(-sigma_m_eff))+a*(log(-sigma_m_eff))*(log(-sigma_m_eff)) + b*(log(-sigma_m_eff)) + c;
    lam_hat = -2*a*(log(-sigma_m_eff));

    eps_v = -lam_hat/(1+e0_hat)*log(-sigma_m_eff);
    e = e0*(1 + eps_v) + eps_v;
    kww = -k0*(1+e0_hat)/lam_hat/gamaw*sigma_m_eff*pow((e/e0),m);
    
    break;
  }
  case kuklik_camclay_mech:{
    //isotropic consolidation according to Kuklik with changing porosity from cam-clay model; 
    //parameters are taken from MEFEL
    if(Tm->nontransq != NULL){
      //tady pokus jestli seje predavani pro Masina:??!!
      s = Tm->givenontransq(saturation_deg, ipp); //actual saturation degree
      ds = Tm->givenontransq(der_saturation_deg, ipp); //actual derivative of saturation degree
      
      sigma_m_eff_bar = Tm->givenontransq(precons_press, ipp); //actual precosolidation pressure
      sigma_m_eff = Tm->givenontransq(mean_stress_eff, ipp); //actual effective mean stress
      if(sigma_m_eff >= sigma_m_eff_init)//this is for pore pressure (pressure is negative)
	sigma_m_eff = sigma_m_eff_init;//due to stability of computing
    }
    
    //porosity at zero pressure
    e0_bar = (lam + kappa)*log(-sigma_m_eff_bar);
    if(sigma_m_eff >= sigma_m_eff_bar){//swelling line
      eps_v = -kappa/(1+e0_bar)*log(-sigma_m_eff);
      e = -kappa*log(-sigma_m_eff) + e0_bar;//another expression: e = e0_bar*(1 + eps_v) + eps_v;
      kww = -k0*(1+e0_bar)/kappa/gamaw*sigma_m_eff*pow((e/e0),m);
    }
    else{ //NCL
      eps_v = -lam/(1+e0)*log(-sigma_m_eff);
      e = -lam*log(-sigma_m_eff) + e0;//another expression: e = e0*(1 + eps_v) + eps_v;
      kww = -k0*(1+e0)/lam/gamaw*sigma_m_eff*pow((e/e0),m);
    }
    
    /* // alternative computation - quadratic approximation aroung bifurcation point (NCL and swelling line)
       e0_bar = (kappa - lam)*log(-sigma_m_eff_bar) + e0;
       c = e0_bar;
       b = -kappa;
       a = -b*(lam/e0) - c*(lam/e0)*(lam/e0);
       e0_hat = -2*a*(log(-sigma_m_eff))+a*(log(-sigma_m_eff))*(log(-sigma_m_eff)) + b*(log(-sigma_m_eff)) + c;
       lam_hat = -2*a*(log(-sigma_m_eff));
       
       eps_v = -lam_hat/(1+e0_hat)*log(-sigma_m_eff);
       e = e0*(1 + eps_v) + eps_v;
       kww = -k0*(1+e0_hat)/lam_hat/gamaw*sigma_m_eff*pow((e/e0),m);
    */

    break;
  }
  default:{
    print_err ("unknown model type is required", __FILE__, __LINE__, __func__);
    abort();
  }
  } 

  return(kww);
}



/**
   function creates capacity coefficient of the general material
   @param pw - water pressure

   @retval capww - capacity coefficient

   12/9/2008, TKr
*/
double con_awf1mat::get_capww(double pw, long ipp)
{
  double sw,dsw_dpw,n,capww;  
    
  switch (model_type){
  case lewis_and_schrefler:
  case van_genuchten:
  case lewis_and_schrefler_mefel_table:
  case lewis_and_schrefler_mefel:{//Lewis and Schrefler's book
    sw = get_sw(pw,ipp);
    dsw_dpw = get_dsw_dpw(pw,ipp);
    n = give_porosity(ipp);
    if(compress == 1){
      //compressible grains:
      capww = (alpha-n)/ks*sw*(sw + dsw_dpw*pw)+n*sw/kw;
      capww = capww + n*dsw_dpw;//this equation is correct
    }
    else{
      //incompressible grains:
      capww = n*dsw_dpw;//this equation is correct
    }    
    break;
  }
  case gardner_exponential:
  case potts_log_linear:
    //case van_genuchten:
    {
    dsw_dpw = get_dsw_dpw(pw,ipp);
    capww = phi0*dsw_dpw; //incompressible grains:
    break;
  }
  case kuklik_camclay:{//fully saturated isotropic consolidation according to Kuklik with changing porosity 
          //from cam-clay model
    capww = 1.0; 
    break;
  }
  case kuklik_camclay_mech:{//fully saturated isotropic consolidation according to Kuklik with changing porosity 
          //from MEFEL cam-clay model
    capww = 1.0; 
    break;
  }
  default:{
    print_err ("unknown model type is required", __FILE__, __LINE__, __func__);
    abort();
  }
  } 
    
  //fprintf(Outt,"ipp = %ld, sw = %e, pw = %e, dsw_dpw = %e, n = %e, capww = %e\n",ipp,sw,pw,dsw_dpw,n,capww);//debug??!!
  //fflush(Outt);//debug??!!

  return(capww);

}


/**
   function returns coefficient for righ-hand side of the general material
   @param pw - water pressure

   @retval fw - first part for right-hand side for continutiy equation

   12/9/2008, TKr
*/
double con_awf1mat::get_fw1(double pw, long ipp)
{
  double krw,fw1;
  
  fw1 = 0.0;

  switch (model_type){
  case lewis_and_schrefler:
  case lewis_and_schrefler_mefel_table:
  case lewis_and_schrefler_mefel:{//Lewis and Schrefler's book
    krw = get_krw(pw,ipp);
    fw1 = krw*kintr/muw0*rhow;//p. 89
    break;
  }
  case gardner_exponential:
  case potts_log_linear:
  case van_genuchten:{
    break;
  }
  case kuklik_camclay:{//fully saturated isotropic consolidation according to Kuklik with changing porosity 
    //from cam-clay model
    break;
  }
  case kuklik_camclay_mech:{//fully saturated isotropic consolidation according to Kuklik with changing porosity 
    //from MEFEL cam-clay model
    krw = get_kww(pw,ipp);
    fw1 = krw*rhow;
    break;
  }
  default:{
    print_err ("unknown model type is required", __FILE__, __LINE__, __func__);
    abort();
  }
  }    
  return(fw1);

}



/**
   function returns coefficient for righ-hand side of the general material
   @param pw - water pressure

   @retval fwu - volumetric strain rate effect on pore water pressure; only for partially coupled METR version

   16/05/2018, TKr
*/
double con_awf1mat::get_fwu(double pw, long ipp)
{
  double sw,fwu,epsv_r,depsv_r,dsr_depsv,n;
  
  fwu = 0.0;
  
  switch (model_type){
  case lewis_and_schrefler:{
    break;
  }
  case lewis_and_schrefler_mefel_table:{
    break;
  }
  case lewis_and_schrefler_mefel:{//Lewis and Schrefler's book
    n = give_porosity(ipp);
    depsv_r = Tm->givenontransq(strain_vol_rate, ipp);      //actual rate of volumetric strain from MEFEL
    dsr_depsv = Tm->givenontransq(der_saturation_deg_depsv, ipp); //actual derivative of saturation degree with respect to volumetric strain
    Tm->ip[ipp].eqother[0] = dsr_depsv;
    Tm->ip[ipp].eqother[1] = depsv_r;
    
    sw = get_sw(pw,ipp);
    fwu = -alpha*sw*depsv_r;//volumetric strain effect
    fwu = fwu - n*dsr_depsv*depsv_r;
    fwu = 0.0; //debug??!!
    break;
  }
  case van_genuchten:{
    //new: debug:??!!
    /* if (Tp->jstep < 0){
    //provissionally debug for pc version with elastic material model
    epsv_r = Tm->givenontransq(strain_vol_rate, ipp);      //actual rate of volumetric strain from MEFEL
    depsv_r = epsv_r - Tm->ip[ipp].eqother[0];
    Tm->ip[ipp].eqother[0] = epsv_r;
    Tm->ip[ipp].eqother[1] = depsv_r;
    }
    else{
    depsv_r = Tm->ip[ipp].eqother[1];
    }
    
    sw = get_sw(pw,ipp);
    fwu = -alpha*sw*depsv_r;
    
    //fprintf(Outt,"ipp = %ld, depsv_r = %e\n",ipp,depsv_r);//debug??!!
    //fflush(Outt);//debug??!!
    */
    //originally:
    fwu = 0.0;
    break;
  }
  case gardner_exponential:
  case potts_log_linear:
  case kuklik_camclay:
  case kuklik_camclay_mech:{
    break;
  }
  default:{
    print_err ("unknown model type is required", __FILE__, __LINE__, __func__);
    abort();
  }
  }    
  return(fwu);

}



/**
   function computes new transmission coefficient (for transmission_vector) 
   for boundary condition (third kind of boundary condition)

   @param trc     - prescribed transmission coefficient on the boundary
   @param ri      - row index
   @param ci      - column index
   @param nn      - number of node
   @param bc      - type of boundary condition
   @param ipp     - number of first integration point on element
*/
double con_awf1mat::transmission_transcoeff(double trc,long ri,long ci,long nn,long bc,long ipp)
{
  long k;
  double c,pw;
  c = 0.0;
  
  pw = nodalval (nn,0);
  
  //  if((ri == 0) && (ci == 0))
  c = get_transmission_transcoeff_ww(pw,bc,ipp);// *scale_pw;//scaling

  c = c*trc;

  return (c);
}


/**
   function computes new transmission coefficient (for transmission_vector) 
   for boundary condition (third kind of boundary condition)

   @param trc     - prescribed transmission coefficient on the boundary
   @param ri      - row index
   @param ci      - column index
   @param nn      - number of node
   @param bc      - type of boundary condition
   @param ipp     - number of first integration point on element
*/
double con_awf1mat::transmission_transcoeff(double trc,long ri,long ci,long nn,long bc,long ipp, int flag)
{
  long k;
  double c,pw;
  c = 0.0;
  
  pw = nodalval (nn,0);

  //  if((ri == 0) && (ci == 0))
  c = get_transmission_transcoeff_ww(pw,bc,ipp,flag);// *scale_pw;//scaling
  
  c = c*trc;
  
  return (c);
}

/**
   function computes new nodal value (for transmission_vector) 
   for boundary condition (third kind of boundary condition)

   @param nodval     - prescribed nodal value
   @param trc2       - second prescribed transmission coefficient on the boundary, 
                       if is needed (for example heat radiation coef.)
   @param ri         - row index
   @param ci         - column index
   @param nn         - number of node
   @param bc         - type of boundary condition
   @param ipp        - number of first integration point on element
*/
double con_awf1mat::transmission_nodval(double nodval,double trc2,long ri,long ci,long nn,long bc,long ipp)
{
  long k;
  double c,pw;
  c = 0.0;
  
  pw = nodalval (nn,0);

  //if((ri == 0) && (ci == 0))
  c = get_transmission_nodval_ww(nodval,pw,bc,ipp);// *scale_pw;//scaling

  return (c);
}


/**
   function computes flux (for transmission_vector) 
   for boundary condition (third kind of boundary condition)

   @param nodval     - prescribed nodal value
   @param trc2       - second prescribed transmission coefficient on the boundary, 
                       if is needed (for example heat radiation coef.)
   @param ri         - row index
   @param ci         - column index
   @param nn         - number of node
   @param bc         - type of boundary condition
   @param ipp        - number of first integration point on element
*/
double con_awf1mat::transmission_flux(double nodval,double trc2,long ri,long ci,long nn,long bc,long ipp)
{
  long k;
  double c,pw;
  c = 0.0;
  
  pw = nodalval (nn,0);

  //if((ri == 0) && (ci == 0))
  c = get_transmission_flux_ww(nodval,pw,bc,ipp);// *scale_pw;//scaling

  return (c);
}



/**
   function creates correct transfer coefficient on the boundary (transmission) for pw

   @param pw - pore water pressure on the boundary
   @param bc - type of boundary condition
   @param ipp - number of first integration point on element
*/
double con_awf1mat::get_transmission_transcoeff_ww(double pw,long bc,long ipp)
{
  double trc;
  
  //water pressure check
  waterpress_check(pw,ipp);

  //other conditions types will be added according to the neeed
  switch (bc){//type of prescribed variable
  case 30:{//pore water pressure

    trc = 1.0;
    
    break;
  }
  case 40:{//simulation of free soil surface
    if(pw < pw_bc)
      trc = 0.0;
    else
      trc = 1.0e+03;
    break;
  }
  default:{
    print_err ("no real boundary condition is prescribed", __FILE__, __LINE__, __func__);
    exit(0);
  }
  }
  return(trc);
}



/**
   function creates correct transfer coefficient on the boundary (transmission) for pw

   @param pw - pore water pressure on the boundary
   @param bc - type of boundary condition
   @param ipp - number of first integration point on element
*/
double con_awf1mat::get_transmission_transcoeff_ww(double pw,long bc,long ipp,int flag)
{
  double trc;
  
  //water pressure check
  waterpress_check(pw,ipp);

  //other conditions types will be added according to the neeed
  switch (bc){//type of prescribed variable
  case 30:{//pore water pressure
    
    trc = 1.0;
    
    break;
  }
  case 33:{//pore water pressure - flux
    if(flag == 1)
      trc=0.0;//into right hand side (matrix)
    else{
      trc=1.0;//into left hand side (flux)
    }
    break;
  }
  case 40:{//simulation of free soil surface
    if(pw < pw_bc)
      trc = 0.0;
    else
      trc = 1.0e+03;
    break;
  }
  default:{
    print_err ("no real boundary condition is prescribed", __FILE__, __LINE__, __func__);
    exit(0);
  }
  }
  return(trc);
}

/**
   function creates correct new nodal value on the boundary (transmission) for pw

   @param bv - prescribed value near the boundary
   @param pw - actual pore water pressure on the boundary
   @param bc - type of boundary condition
   @param ipp - number of first integration point on element
*/
double con_awf1mat::get_transmission_nodval_ww(double bv,double pw,long bc,long ipp)
{
  double new_nodval;
  
  //water pressure check
  waterpress_check(pw,ipp);

  switch (bc){//type of prescribed variable
  case 30:{//pore water pressure
    new_nodval = bv;
    break;
  }
  case 33:{//pore water pressure - flux
    new_nodval = bv - pw;//minus sign
    break;
  }
  case 40:{//simulation of free soil surface
    new_nodval = bv;
    break;
  }
  default:{
    print_err ("no real boundary condition is prescribed", __FILE__, __LINE__, __func__);
    exit(0);
  }
  }
  return(new_nodval);
}



/**
   function creates flux on the boundary (transmission - convective mass transfer) for pw

   @param bv - prescribed value near the boundary
   @param pw - actual pore water pressure on the boundary
   @param bc - type of boundary condition
   @param ipp - number of first integration point on element
*/
double con_awf1mat::get_transmission_flux_ww(double bv,double pw,long bc,long ipp)
{
  double flux,trc;
  
  //water pressure check
  waterpress_check(pw,ipp);

  switch (bc){//type of prescribed variable
  case 30:{//pore water pressure
    flux = bv - pw;//minus sign
    break;
  }
  case 33:{//pore water pressure - flux
    flux = bv - pw;//minus sign
    break;
  }
  case 40:{//simulation of free soil surface
    if(pw < pw_bc)
      trc = 0.0;
    else
      trc = 1.0e+03;
    flux = trc*(bv - pw);//minus sign
    break;
  }
  default:{
    print_err ("no real boundary condition is prescribed", __FILE__, __LINE__, __func__);
    exit(0);
  }
  }

  return(flux);
}




/**
   function computes all other variables at nodes
   @param compother - number of other components
   @param pw - water capillary pressure on actual node
   @param ipp - first integration point on element

   @retval other - other variable

   03/03/2011, TKr
*/

double con_awf1mat::get_othervalue(long compother,double pw, long ipp)
{
  double other;
  state_eq tt;

  switch (compother){
  case 0:{//capillary pressure
    //other = -pw;
    other = pw/mefel_units;
    break;
  }
  case 1:{//saturation
    other = get_sw(pw,ipp);
    break;
  }
  case 2:{//specific moisture content
    other = get_w(pw,ipp);
    //other = Tm->ip[ipp].eqother[0];  //dsr_depsv//debug
    //other = Tm->ip[ipp].eqother[1];  //depsv_r//debug
    //Tm->givenontransq(strain_vol_rate, ipp);//debug
    break;
  }    
  default:{
    print_err("unknown type of component is required in function", __FILE__, __LINE__, __func__);
    abort();
  }
  }
  return (other);

}



/**
     function prints names of all other variables at nodes
     @param out - output rhle
     @param compother - number of other components

     03/03/2011, TKr
*/
void con_awf1mat::print_othervalue_name(FILE *out,long compother)
{
  switch (compother){
  case 0:{//capillary pressure
    fprintf (out,"Capillary pressure (Pa)");
    break;
  }
  case 1:{//saturation
    fprintf (out,"Degree of saturation ()       ");
    break;
  }
  case 2:{//moisture content
    fprintf (out,"Moisture (water) content (kg/kg)      ");
    //fprintf (out,"Moisture content (kg/kg)      ");
    break;
  }    
  default:{
    print_err ("unknown type of component is required in function", __FILE__, __LINE__, __func__);
    abort();
  }
  }
}



/**
   function checks if computed unknowns are physically reasonable
   @param nv  - vector of unknowns
   @param ipp - number of integration point


   28/02/2011, TKr
*/
void con_awf1mat::values_correction (vector &nv, long ipp)
{
  //  pore water pressure control
  waterpress_check(nv[0],ipp);
}


/**
   function checks if water pressure is non-positive
   @param pw - pore water pressure
   @param ipp - number of integration point


   28/02/2011, TKr
*/
void con_awf1mat::waterpress_check(double &pw,long ipp)
{
  //this must be checked:
  //if (pw >= 0.0) 
  //pw = -1.0;
  
  //storing into gauss point
  //Tm->ip[ipp].av[0] = pw;
}

/**
  This function updates values in the other array reached in the previous equlibrium state to
  values reached in the new actual equilibrium state.

  @param ipp - integration point number in the mechmat ip array.

   24/08/2012, TKr
*/
void con_awf1mat::updateval (long ipp)
{
}


/**
   This function initializes material model data
   with respect of consistency parameter gamma.

   @param ipp - integration point number

   24/08/2012, TKr
*/
void con_awf1mat::initval(long ipp)
{
}



/**
  The function returns ordered dof names of primary unknowns 
  required by the model.

  @param dofname   - array of uknown name for particular nodal dofs (output)
                     dofname[i] = name of i-th nodal unknown (for names see aliast.h - enum namevart)
  @param ntm       - number of transported media = number of nodal dof = length of array dofname

  Created by Tomas Krejci according to Tomas Koudelka, 28/04/2014
*/
void con_awf1mat::give_dof_names(namevart *dofname, long ntm)
{
  if (ntm < 1)
  {
    print_err("the model defines %ld unknowns but number of transported media is %ld", 
              __FILE__, __LINE__, __func__, 1, ntm);
    abort();
  }
  dofname[0] = trf_water_press;
}




/**
   function returns water pressure

   @param ipp - integration point number

   @retval pw - pore water pressure
*/
double con_awf1mat::give_water_pressure(long ipp)
{
  double pw;

  pw = Tm->ip[ipp].av[0];

  return(pw);
}


/**
   function computes suction stress s = -pw = pc; for MEFEL stress notation s = pw = -pc

   @param ipp - integration point number

   @retval suction - suction stress [Pa]

   25/9/2012, TKr
*/
double con_awf1mat::give_suction(long ipp)
{
  double pw,suction;
  
  pw = Tm->ip[ipp].av[0];

  switch (model_type){
  case lewis_and_schrefler:{//Lewis and Schrefler's book
    suction = -pw;//this is correct for Lewis and Schrefler's notation
    break;
  }
  case lewis_and_schrefler_mefel_table:
  case lewis_and_schrefler_mefel:{//Lewis and Schrefler's book, saturation degree ind its derivative are obtained from mefel;
    suction = pw;//this is correct, because capillary water pressure is negative
    
    suction = suction/mefel_units;//corrected units for mefel //basic units = Pa    
    break;
  }
  case gardner_exponential:
  case potts_log_linear:
  case van_genuchten:
  case kuklik_camclay:
  case kuklik_camclay_mech:{//isotropic consolidation according to Kuklik with changing porosity from MEFEL cam-clay model
    suction = -pw;
    break;
  }
  default:{
    print_err ("unknown model type is required", __FILE__, __LINE__, __func__);
    abort();
  }
  } 
  return(suction);
}


/**
   function returns the degree of saturation
   @param ipp - integration point number

   @retval saturation degree [-]

   25/9/2012, TKr
*/
double con_awf1mat::give_saturation_degree(long ipp)
{
  double pw,s;
  
  pw = Tm->ip[ipp].av[0];
  s = get_sw(pw,ipp);
  
  return(s);
}

/**
   function computes water content w [kg/kg]
   @param pw - water capillary pressure

   @retval w - computes water content w [kg/kg]
*/
double con_awf1mat::get_w(double pw,long ipp)
{
  double w,n,s;

  n = give_porosity(ipp);
  s = get_sw(pw,ipp);

  w = (n*s*rhow)/(1.0 - n)/rhos0;

  return(w);
}


/**
  The funtion marks required non-transport quantities in the array antq.

  @param antq - array with flags for used material types
                antq[i] = 1 => quantity type nontransquant(i+1) is required
                antq[i] = 0 => quantity type nontransquant(i+1) is not required

  @return The function does not return anything, but it may change content of antq array.
*/
void con_awf1mat::give_reqntq(long *antq)
{
  switch (model_type){
  case lewis_and_schrefler:{//Lewis and Schrefler's book
    break;
  }
  case lewis_and_schrefler_mefel_table:
  case lewis_and_schrefler_mefel:{//Lewis and Schrefler's book, saturation degree ind its derivative are obtained from mefel;
    antq[saturation_deg-1] = 1;
    antq[der_saturation_deg-1] = 1;
    antq[der_saturation_deg_depsv-1] = 1;
    antq[porosity-1] = 1;
    antq[strain_vol_rate-1] = 1;
    break;
  }  
  case van_genuchten:{
    antq[strain_vol_rate-1] = 1;//provissionally for pc version testing//debug??!!
    break;
  }
  case gardner_exponential:
  case potts_log_linear:
  case kuklik_camclay:{//isotropic consolidation according to Kuklik with changing porosity from cam-clay model
    break;
  }
  case kuklik_camclay_mech:{//isotropic consolidation according to Kuklik with changing porosity from MEFEL cam-clay model
    antq[precons_press-1] = 1;
    antq[mean_stress_eff-1] = 1;
    antq[saturation_deg-1] = 1;
    antq[der_saturation_deg-1] = 1;
    break;
  }
  default:{
    print_err ("unknown model type is required", __FILE__, __LINE__, __func__);
    abort();
  }
  } 
}
