/*
  File:             consol_hwf2.cpp
  Author:           Tomas Krejci, 06/04/2018
  Purpose:          computes conductivity and capacity matrices in a material point for consol_hwf2 porous media;
                    material model for saturated-nonsaturated water flow and heat transfer 
                    in a deforming porous medium (soils)
  unknowns:         number of unknowns=2, pw = pore water pressure, t = temperature
  sources:          

  1. THE FINITE ELEMENT METHOD IN THE STATIC AND D?YNAMIC DEFORMATION AND CONSOLIDATION OF POROUS MEDIA
  R. W. Lewis, B.A. Schrefler
  
  2. NONLINEAR MODELLING OF CONCRETE AS POROUS MATERIAL IN HIGH TEMPERATURE CONDITIONS
  Francesco Pesavento - doctoral thesis                      
  
*/ 

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "aliast.h"
#include "consol_hwf2.h"
#include "globalt.h"
#include "C60bazant.h" //testing
#include "C60baroghel.h" //testing
#include "globmatt.h"

con_hwf2mat::con_hwf2mat()
{
  compress = 0; //compressible grains: 0=no; 1=yes

  //PHYSICAL PROPERTIES OF SOIL set to zero
  alpha0 = 0.0;
  ks0 = 0.0;
  phi0 = 0.0;
  kintr0 = 0.0;
  betas0 = 0.0;
  rhos0 = 0.0;
  cps0 = 0.0;
  lambda0 = 0.0;
  //pro bukov:
  lambda1 = 0.0;
  lambda2 = 0.0;
  sr1 = 0.0;
  sr2 = 0.0;
  

  // PHYSICAL PROPERTIES OF WATER
  kw0 = 2.0e9;//bulk modulus of water
  rhow0 = 1000.0;
  muw0 = 1.0e-3;
  cpw = 4181.0;

  mefel_units = 1.0;//basic units for pressures = Pa (Pascals)
}

con_hwf2mat::~con_hwf2mat()
{}


/**
   function reads parameters
   
   @param in - input file

   20/05/2017, TKr
*/
void con_hwf2mat::read(XFILE *in)
{
  xfscanf (in,"%k%m","heatairwaterflowtype",&heatairwaterflowtype_kwdset, &model_type);
  xfscanf (in,"%d", &compress);

  switch (model_type){
  case lewis_and_schrefler2hw:{//Lewis and Schrefler's model
    xfscanf (in,"%le %le %le %le %le %le %le %le", &alpha0, &ks0, &phi0, &kintr0, &betas0, &rhos0, &cps0, &lambda0);
    lewis_ret.read(in);
    break;
  }
  case lewis_and_schrefler2hw_mefel:{//Lewis and Schrefler's book
    //xfscanf (in,"%le %le %le %le %le %le %le %le %le", &mefel_units, &alpha0, &ks0, &phi0, &kintr0, &betas0, &rhos0, &cps0, &lambda0);
    //pro bukov:
    xfscanf (in,"%le %le %le %le %le %le %le %le %le %le %le %le", &mefel_units, &alpha0, &ks0, &phi0, &kintr0, &betas0, &rhos0, &cps0, &lambda1,&lambda2,&sr1,&sr2);
    break;
  }
  case van_genuchten2hw:{//partially saturated medium =Van Genuchten model
    xfscanf (in,"%le %le %le %le %le %le %le %le %le", &mefel_units, &alpha0, &ks0, &phi0, &kintr0, &betas0, &rhos0, &cps0, &lambda0);
    van_genuchten_ret.read(in);
    break;
  }
  default:{
    print_err ("unknown model type is required", __FILE__, __LINE__, __func__);
    abort();
  }
  } 
}


/**
   function prints parameters
   
   @param out - output file

   20/05/2017, TKr
*/
void con_hwf2mat::print(FILE *out)
{
  fprintf (out,"\n %d ", int(model_type));
  fprintf (out,"\n %d ", compress);

  switch (model_type){
  case lewis_and_schrefler2hw:{//Lewis and Schrefler's model
    fprintf (out,"\n %le %le %le %le %le %le %le %le ",alpha0,ks0,phi0,kintr0,betas0,rhos0,cps0,lambda0);
    lewis_ret.print(out);
    break;
  }
  case lewis_and_schrefler2hw_mefel:{//Lewis and Schrefler's book
    fprintf (out,"\n %le %le %le %le %le %le %le %le %le ",mefel_units,alpha0,ks0,phi0,kintr0,betas0,rhos0,cps0,lambda0);
    break;
  }
  case van_genuchten2hw:{//partially saturated medium =Van Genuchten model
    fprintf (out,"\n %le %le %le %le %le %le %le %le %le ",mefel_units,alpha0,ks0,phi0,kintr0,betas0,rhos0,cps0,lambda0);
    van_genuchten_ret.print(out);
    break;
  }
  default:{
    print_err ("unknown model type is required", __FILE__, __LINE__, __func__);
    abort();
  }
  } 
}

/**
   function computes cubic thermal expansion coefficient of solid (K-1)

   @retval betas -  cubic thermal expansion coefficient of solid (K-1)
*/
double con_hwf2mat::get_betas(long ipp)
{
  double betas;

  betas = betas0;

  return(betas);
}


/**
   function computes specific heat of solid skeleton
   @param t - temperature

   @retval cps - specific heat of soil skeleton
*/
double con_hwf2mat::get_cps(double t)
{
  double cps;

  cps = cps0;

  return(cps);
}


/**
   function computes  volume density of concrete skeleton,
   changes of solid density, caused by dehydratation process

   @param t - temperature

   @retval rhos - volume density of soil skeleton
*/
double con_hwf2mat::get_rhos(double t)
{
  double rhos;
   
  rhos = rhos0;
  
  return(rhos);
}


/**
   function computes degree of saturation(sorption curve)
   @param pw - water pressure
   @param t - temperature

   @retval sw - degree of saturation

   20/05/2017, TKr
*/
double con_hwf2mat::get_sw(double pw, double t, long ipp)
{
  double sw;

  switch (model_type){
  case lewis_and_schrefler2hw:{//Lewis and Schrefler's model
    sw = lewis_ret.sw(pw);
    break;
  }
  case lewis_and_schrefler2hw_mefel:{//saturation degree ind its derivative are obtained from mefel;
    if(Tm->nontransq != NULL){
      sw = Tm->givenontransq(saturation_deg, ipp); //actual saturation degree
    }
    else{
      //this below is for testing:
      sw = Tm->C60bazant[0].sat(-pw,t);
      sw = Tm->C60baroghel[0].sat(-pw,t);
    } 
    break;
  }
  case van_genuchten2hw:{//partially saturated medium = Van Genuchten model
    pw = pw/mefel_units;
    sw = van_genuchten_ret.sw(pw);
    break;
  }    
  default:{
    print_err ("unknown model type is required", __FILE__, __LINE__, __func__);
    abort();
  }
  }  
  
  return(sw);
}


/**
   function computes specific moisture content (partial derivative of degree of saturation with respect to pw)
   @param pw - water pressure
   @param t - temperature

   @retval ds_dpw - specific moisture content = partial derivative of degree of saturation with respect to pw


   20/05/2017, TKr
*/
double con_hwf2mat::get_dsw_dpw(double pw, double t, long ipp)
{
  double dsw_dpw;

  switch (model_type){
  case lewis_and_schrefler2hw:{//Lewis and Schrefler's book - Liakopoulos test
    dsw_dpw = lewis_ret.dsw_dpw(pw);
    break;
  }
  case lewis_and_schrefler2hw_mefel:{//saturation degree ind its derivative are obtained from mefel;
    if(Tm->nontransq != NULL){
      dsw_dpw = Tm->givenontransq(der_saturation_deg, ipp); //actual derivative of saturation degree
      dsw_dpw = dsw_dpw/mefel_units;
    }
    else{
      //this below is for testing:
      dsw_dpw = -Tm->C60bazant[0].dsat_dpc(-pw,t);
      dsw_dpw = -Tm->C60baroghel[0].dsat_dpc(-pw,t);
    }
    break;
  }
  case van_genuchten2hw:{//partially saturated medium = Van Genuchten model
    pw = pw/mefel_units;
    dsw_dpw = van_genuchten_ret.dsw_dpw(pw);
    break;
  }
  default:{
    print_err ("unknown model type is required", __FILE__, __LINE__, __func__);
    abort();
  }
  } 

  return(dsw_dpw);
}


/**
   function computes partial derivative of degree of saturation with respect to t
   @param pw - water pressure
   @param t - temperature

   @retval dsw_dt - partial derivative of degree of saturation with respect to temperature


   20/05/2017, TKr
*/
double con_hwf2mat::get_dsw_dt(double pw, double t, long ipp)
{
  double dsw_dt;
  
  switch (model_type){
  case lewis_and_schrefler2hw:{//Lewis and Schrefler's book
    dsw_dt = lewis_ret.dsw_dt(pw);//p. 167
    break;
  }
  case lewis_and_schrefler2hw_mefel:{//from mefel;
    if(Tm->nontransq != NULL){
      dsw_dt = Tm->givenontransq(der_saturation_deg_dtemp, ipp); //actual derivative of saturation degree
    }
    else{//this below is for testing
      dsw_dt = Tm->C60bazant[0].dsat_dt(-pw,t);
      dsw_dt = Tm->C60baroghel[0].dsat_dt(-pw,t);
    }
    break;
  }
  case van_genuchten2hw:{//partially saturated medium = Van Genuchten model
    pw = pw/mefel_units;
    dsw_dt = van_genuchten_ret.dsw_dt(pw);
    break;
  }
  default:{
    print_err ("unknown model type is required", __FILE__, __LINE__, __func__);
    abort();
  }
  } 
  
  return(dsw_dt);
}


/**
   function computes intrinsic permeability
   @param pw - water pressure
   @param t - temperature
   
   @retval kintr - intrinsic permeability

   27/07/2017, TKr
*/
double con_hwf2mat::get_kintr(double pw, double t, long ipp)
{
  double kintr;

  kintr = kintr0;

  return(kintr);
}

/**
   function computes water relative permeability
   @param pw - water pressure
   @param t - temperature
   
   @retval krw - water relative permeability

   20/05/2017, TKr
*/
double con_hwf2mat::get_krw(double pw, double t, long ipp)
{
  double krw,sw;

  krw=1.0;

  switch (model_type){
  case lewis_and_schrefler2hw:{//Lewis and Schrefler's book
    sw = lewis_ret.sw(pw);
    krw = lewis_ret.krw(sw);
    break;
  }
  case lewis_and_schrefler2hw_mefel:{//from mefel;
    krw = 1.0;
    break;
  }
  case van_genuchten2hw:{//partially saturated medium = Van Genuchten model
    pw = pw/mefel_units;
    krw = van_genuchten_ret.krw(pw);
    break;
  }
  default:{
    print_err ("unknown model type is required", __FILE__, __LINE__, __func__);
    abort();
  }
  }

  return(krw);
}



/**
   function computes Biot's constant
   @param pw - pore water pressure
   @param t - temperature
   
   @retval alpha - Biot's constant
*/
double con_hwf2mat::get_alpha(double pw, double t,long ipp)
{
  double alpha;
  //double kt,ks;
  
  //kt = get_kt(pw,t,ipp);
  //ks = get_ks(pw,t,ipp);
  
  //alpha = 1.0 - kt/ks;  

  alpha = alpha0;//provisionally
  
  return(alpha);
}

/**
   function computes bulk modulus of solid phase
   @param pw - pore water pressure
   @param t - temperature

   @retval ks - bulk modulus of solid phase
*/
double con_hwf2mat::get_ks(double pw,double t,long ipp)
{
  double ks;

  ks = ks0;

  return(ks);
}


/**
   function computes water density
   @param t - temperature

   @retval rhow - water density
*/
double con_hwf2mat::get_rhow(double t)
{
  double rhow;

  rhow = rhow0;//provisionally
  
  return(rhow);
}


/**
   function computes compresibility coefficient of water
   
   @retval kw - compresibility coefficient of water

*/
double con_hwf2mat::get_kw(double pw,double t,long ipp)
{
  double kw;
  
  kw = kw0;

  return(kw);
}


/**
   function volume computes thermal expansion coefficient of solid - water for compressible solid grains
   @param pw - pore water pressure
   @param t - temperature

   @retval betasw - volume thermal expansion coefficient of solid - water for compressible solid grains
*/
double con_hwf2mat::get_betasw(double pw,double t,long ipp)
{
  double alpha,betasw,betas,n,sw,betaw;

  alpha = get_alpha(pw,t,ipp);
  betas = get_betas(ipp);
  n = get_porosity(pw,t,ipp);
  sw = get_sw(pw,t,ipp);
  betaw = get_betaw(t);

  betasw = sw*(betas*(alpha - n) + n*betaw);

  return(betasw);
}


/**
   function computes dynamic viscosity of water = 1000e-6 Pa*s at 20 C
   @param t - temperature

   @retval muw dynamic viscosity of water = 1000e-6 Pa*s at 20 deg. C
*/
double con_hwf2mat::get_muw(double t)
{
  double muw;

  muw = muw0;

  return(muw);
}

/**
   function computes water specific heat

   @retval cpw - water specific heat
*/
double con_hwf2mat::get_cpw()
{
  double c;

  c = cpw;

  return(c);
}

/**
   function computes volume thermal expansion coefficient of water
   @param t - temperature
   
   @retval betaw - volume thermal expansion coefficient of water
*/
double con_hwf2mat::get_betaw(double t)
{
  double betaw;
  
  betaw = 0.68e-4;//[K-1] at t=273.15
  //betaw = 10.1e-4;//[K-1] at t=420.0
  
  //temporarily??!!
  betaw = 0.63e-5;
  //betaw = 0.68 + (10.1 - 0.68)/(420.0 - 273.15)*(t - 273.15);//linear expression (rough)
  
  return(betaw);
}


/**
   function computes effective thermal capacity of partially saturated medium
   @param pw - pore water pressure
   @param t - temperature
   @param ipp - number of integration point

   @retval rhocp - effective thermal capacity of partially saturated medium
*/
double con_hwf2mat::get_rhocp(double pw,double t,long ipp)
{
  double rhocp;

  rhocp = rhos0*cps0;

  return(rhocp);
}



/**
   function computes effective thermal conductivity of partially saturated concrete
   @param pw - pore water pressure
   @param t - temperature

   @retval lambdaeff - effective thermal conductivity of partially saturated soil
*/
double con_hwf2mat::get_lambdaeff(double pw,double t,long ipp)
{
  double lambdaeff,s;
  
  switch (model_type){
  case lewis_and_schrefler2hw:{//Lewis and Schrefler's model
    lambdaeff = lambda0;
    break;
  }
  case lewis_and_schrefler2hw_mefel:{//Lewis and Schrefler's book
    s = get_sw(pw,t,ipp);
    lambdaeff = (lambda2-lambda1)/(sr2-sr1)*s;
    break;
  }
  case van_genuchten2hw:{//partially saturated medium =Van Genuchten model
    lambdaeff = lambda0;
    break;
  }
  default:{
    print_err ("unknown model type is required", __FILE__, __LINE__, __func__);
    abort();
  }
  }
  return(lambdaeff);
}


/**
   function computes conductivity matrix of the material
   in the required integration point
   
   @param d - conductivity %matrix of material
   @param ri  - row index
   @param ci  - column index   
   @param ipp - number of integration point
   
*/
void con_hwf2mat::matcond (matrix &d,long ri,long ci,long ipp)
{
  long n;
  n = d.n;
  
  switch (n){
  case 1:{
    matcond1d (d,ri,ci,ipp);//1D
    break;
  }
  case 2:{
    matcond2d (d,ri,ci,ipp);//2D
    break;
  }
  case 3:{
    matcond3d (d,ri,ci,ipp);//3D
    break;
  }
  default:{
    print_err ("unknown number of components of stress tensor is required", __FILE__, __LINE__, __func__);
    abort();
  }
  }
}


/**
   function creates conductivity matrix of the material for 1D problems
   
   @param d   - conductivity %matrix of the material
   @param ri  - row index
   @param ci  - column index   
   @param ipp - number of integration point
*/
void con_hwf2mat::matcond1d (matrix &d,long ri,long ci,long ipp)
{
  double k;
  double pw,t;
  k = 0.0;
  
  pw = Tm->ip[ipp].av[0];// *scale_pw;//scaling
  t = Tm->ip[ipp].av[1];// *scale_t;//scaling
  
  if((ri == 0) && (ci == 0))
    k = get_kww(pw,t,ipp);// *scale_pw;//scaling
  if((ri == 0) && (ci == 1))
    k = get_kwt(pw,t,ipp);// *scale_g;//scaling
  
  if((ri == 1) && (ci == 0))
    k = get_ktw(pw,t,ipp);// *scale_pw;//scaling
  if((ri == 1) && (ci == 1))
    k = get_ktt1(pw,t,ipp);// *scale_g;//scaling
  
  d[0][0] = k;
}

/**
   function creates conductivity matrix of the material for 2D problems
   
   @param d   - conductivity %matrix of the material
   @param ri  - row index
   @param ci  - column index   
   @param ipp - number of integration point
*/
void con_hwf2mat::matcond2d (matrix &d,long ri,long ci,long ipp)
{
  double k;
  double pw,t;
  k = 0.0;
  
  pw = Tm->ip[ipp].av[0];// *scale_pw;//scaling
  t = Tm->ip[ipp].av[1];// *scale_t;//scaling
  
  if((ri == 0) && (ci == 0))
    k = get_kww(pw,t,ipp);// *scale_pw;//scaling
  if((ri == 0) && (ci == 1))
    k = get_kwt(pw,t,ipp);// *scale_g;//scaling
  
  if((ri == 1) && (ci == 0))
    k = get_ktw(pw,t,ipp);// *scale_pw;//scaling
  if((ri == 1) && (ci == 1))
    k = get_ktt1(pw,t,ipp);// *scale_g;//scaling
  
  fillm(0.0,d);
  
  d[0][0] = k;   d[0][1] = 0.0;
  d[1][0] = 0.0; d[1][1] = k;
}

/**
   function creates conductivity matrix of the material for 3D problems
   
   @param d   - conductivity %matrix of the material
   @param ri  - row index
   @param ci  - column index   
   @param ipp - number of integration point
*/

void con_hwf2mat::matcond3d (matrix &d,long ri,long ci,long ipp)
{
  double k;
  double pw,t;
  k = 0.0;
  
  pw = Tm->ip[ipp].av[0];// *scale_pw;//scaling
  t = Tm->ip[ipp].av[1];// *scale_t;//scaling
  
  if((ri == 0) && (ci == 0))
    k = get_kww(pw,t,ipp);// *scale_pw;//scaling
  if((ri == 0) && (ci == 1))
    k = get_kwt(pw,t,ipp);// *scale_g;//scaling
  
  if((ri == 1) && (ci == 0))
    k = get_ktw(pw,t,ipp);// *scale_pw;//scaling
  if((ri == 1) && (ci == 1))
    k = get_ktt1(pw,t,ipp);// *scale_g;//scaling
  
  fillm(0.0,d);
  
  d[0][0]=k;   d[0][1]=0.0; d[0][2]=0.0;
  d[1][0]=0.0; d[1][1]=k;   d[1][2]=0.0;
  d[2][0]=0.0; d[2][1]=0.0; d[2][2]=k;
}


/**
   function creates capacity matrix of the material
   
   @param c   - capacity matrix of the material
   @param ri  - row index
   @param ci  - column index   
   @param ipp - number of integration point
*/
void con_hwf2mat::matcap (double &c,long ri,long ci,long ipp)
{
  double pw,t;
  c = 0.0;
  
  pw = Tm->ip[ipp].av[0];// *scale_pw;//scaling
  t  = Tm->ip[ipp].av[1];// *scale_t;//scaling
  
  if((ri == 0) && (ci == 0))
    c = get_capww(pw,t,ipp);// *scale_pw;//scaling
  if((ri == 0) && (ci == 1))
    c = get_capwt(pw,t,ipp);// *scale_t;//scaling
  
  if((ri == 1) && (ci == 0))
    c = get_captw(pw,t,ipp);// *scale_pw;//scaling
  if((ri == 1) && (ci == 1))
    c = get_captt(pw,t,ipp);// *scale_g;//scaling

  check_math_errel(0);

}



/**
   function computes conductivity matrix of the material
   in the required integration point
   
   @param d - conductivity %matrix of material
   @param ri  - row index
   @param ci  - column index   
   @param ipp - number of integration point

*/
void con_hwf2mat::matcond2 (matrix &d,long ri,long ci,long ipp)
{
  long n;
  n = d.n;

  switch (n){
  case 1:{
    matcond1d_2 (d,ri,ci,ipp);//1D
    break;
  }
  case 2:{
    matcond2d_2 (d,ri,ci,ipp);//2D
    break;
  }
  case 3:{
    matcond3d_2 (d,ri,ci,ipp);//3D
    break;
  }
  default:{
    print_err ("unknown number of components of stress tensor is required", __FILE__, __LINE__, __func__);
    abort();
  }
  }
}




/**
   function creates conductivity matrix of the material for 1D problems (convective term)

   @param d   - conductivity %matrix of the material
   @param ri  - row index
   @param ci  - column index   
   @param ipp - number of integration point
*/
void con_hwf2mat::matcond1d_2 (matrix &d,long ri,long ci,long ipp)
{
  double a,c;
  double pw,t;
      
  pw = Tm->ip[ipp].av[0];// *scale_pw;//scaling
  t  = Tm->ip[ipp].av[1];// *scale_t;//scaling
  
  fillm(0.0,d);
  
  if((ri == 1) && (ci == 0)){
  }

  if((ri == 1) && (ci == 1)){
    a = get_ktt2a(pw,t,ipp);// *scale_t;//scaling
    c = get_ktt2c(pw,t,ipp);// *scale_t;//scaling
    
    d[0][0] = -1.0*a*Tm->ip[ipp].grad[0][0] + (a*c)*Tp->gr[0];
  }  
}


/**
   function creates conductivity matrix of the material for 1D problems (convective term)

   @param d   - conductivity %matrix of the material
   @param ri  - row index
   @param ci  - column index   
   @param ipp - number of integration point
*/
void con_hwf2mat::matcond2d_2 (matrix &d,long ri,long ci,long ipp)
{
  double a,c;
  double pw,t;
  
  pw = Tm->ip[ipp].av[0];// *scale_pw;//scaling
  t  = Tm->ip[ipp].av[1];// *scale_t;//scaling
  
  fillm(0.0,d);
  
  if((ri == 1) && (ci == 0)){
  }  
  if((ri == 1) && (ci == 1)){
    a = get_ktt2a(pw,t,ipp);// *scale_t;//scaling
    c = get_ktt2c(pw,t,ipp);// *scale_t;//scaling
    
    d[0][0] = -1.0*a*Tm->ip[ipp].grad[0][0] + (a*c)*Tp->gr[0];
    d[0][1] = -1.0*a*Tm->ip[ipp].grad[0][1] + (a*c)*Tp->gr[1];
  }
}



/**
   function creates conductivity matrix of the material for 1D problems (convective term)

   @param d   - conductivity %matrix of the material
   @param ri  - row index
   @param ci  - column index   
   @param ipp - number of integration point
*/
void con_hwf2mat::matcond3d_2 (matrix &d,long ri,long ci,long ipp)
{
  double a,c;
  double pw,t;
  
  pw = Tm->ip[ipp].av[0];// *scale_pw;//scaling
  t  = Tm->ip[ipp].av[1];// *scale_t;//scaling
  
  fillm(0.0,d);
    
  if((ri == 1) && (ci == 0)){
  }  
  if((ri == 1) && (ci == 1)){
    a = get_ktt2a(pw,t,ipp);// *scale_t;//scaling
    c = get_ktt2c(pw,t,ipp);// *scale_t;//scaling
    
    d[0][0] = -1.0*a*Tm->ip[ipp].grad[0][0] + (a*c)*Tp->gr[0];
    d[0][1] = -1.0*a*Tm->ip[ipp].grad[0][1] + (a*c)*Tp->gr[1];
    d[0][2] = -1.0*a*Tm->ip[ipp].grad[0][2] + (a*c)*Tp->gr[2];
  }
}



/**
   function computes volume part of right-hand side matrix
   in the required integration point
   
   @param d - right-hand side %matrix of material
   ...
   @param ipp - number of integration point
   
   11.2.2003
*/
void con_hwf2mat::rhs_volume (matrix &d,long ri,long ci,long ipp)
{
  long m;
  m = d.m;
  
  switch (m){
  case 1:{
    rhs1d1 (d,ri,ci,ipp);//1D
    break;
  }
  case 2:{
    rhs2d1 (d,ri,ci,ipp);//2D
    break;
  }
  case 3:{
    rhs3d1 (d,ri,ci,ipp);//3D
    break;
  }
  default:{
    print_err ("unknown number of components of stress tensor is required", __FILE__, __LINE__, __func__);
    abort();
  }
  }
}



/**
   function computes volume part 2 of right-hand side matrix
   in the required integration point
   
   @param cc - right-hand side coefficient of material
   @param ri  - row index
   @param ci  - column index   
   @param ipp - number of integration point
*/
void con_hwf2mat::rhs_volume2 (double &cc,long ri,long ci,long ipp)
{
  double pw,t;
  
  pw = Tm->ip[ipp].av[0];
  t = Tm->ip[ipp].av[1];

  if(ri == 0){
    cc = get_fwu(pw,t,ipp);
  }
  if(ri == 1){
    cc = 0.0;
  }
}



/**
   function creates volume right-hand side matrix of the material for 1D problems
   @param d - right-hand %matrix of the material
   ...
   @param ipp - number of integration point
*/
void con_hwf2mat::rhs1d1 (matrix &d,long ri,long ci,long ipp)
{
  double f,pw,t;
  
  pw = Tm->ip[ipp].av[0];// *scale_pw;//scaling
  t  = Tm->ip[ipp].av[1];// *scale_t;//scaling
  
  if(ri == 0){
    f = get_fw1(pw,t,ipp);
    
    fillm(0.0,d);
    d[0][0] = f*Tp->gr[0];
  }
  if(ri == 1){
    f = get_ft1(pw,t,ipp);
    
    fillm(0.0,d);
    d[0][0] = f*Tp->gr[0];
  }
}

/**
   function creates volume right-hand side matrix of the material for 2D problems
   @param d - right-hand %matrix of the material
   ...
   @param ipp - number of integration point
*/
void con_hwf2mat::rhs2d1 (matrix &d,long ri,long ci,long ipp)
{
  double f,pw,t;
  
  pw = Tm->ip[ipp].av[0];// *scale_pw;//scaling
  t  = Tm->ip[ipp].av[1];// *scale_t;//scaling
  
  if(ri == 0){
    f = get_fw1(pw,t,ipp);
    
    fillm(0.0,d);
    d[0][0] = f*Tp->gr[0];
    d[1][0] = f*Tp->gr[1];
  }
  if(ri == 1){
    f = get_ft1(pw,t,ipp);
    
    fillm(0.0,d);
    d[0][0] = f*Tp->gr[0];
    d[1][0] = f*Tp->gr[1];
  }
}

/**
   function creates volume right-hand side matrix of the material for 3D problems
   @param d - right-hand %matrix of the material
   ...
   @param ipp - number of integration point
*/
void con_hwf2mat::rhs3d1 (matrix &d,long ri,long ci,long ipp)
{
  double f,pw,t;

  pw = Tm->ip[ipp].av[0];// *scale_pw;//scaling
  t  = Tm->ip[ipp].av[1];// *scale_t;//scaling

  if(ri == 0){
    f = get_fw1(pw,t,ipp);

    fillm(0.0,d);
    d[0][0] = f*Tp->gr[0];
    d[1][0] = f*Tp->gr[1];
    d[2][0] = f*Tp->gr[2];
  }
  if(ri == 1){
    f = get_ft1(pw,t,ipp);
    
    fillm(0.0,d);
    d[0][0] = f*Tp->gr[0];
    d[1][0] = f*Tp->gr[1];
    d[2][0] = f*Tp->gr[2];
  }
}




/**
   function computes new transmission coefficient (for transmission_vector) 
   for boundary condition (third kind of boundary condition)

   @param trc     - prescribed transmission coefficient on the boundary
   @param ri      - row index
   @param ci      - column index
   @param nn      - number of node
   @param bc      - type of boundary condition
   @param ipp     - number of first integration point on element
*/
double con_hwf2mat::transmission_transcoeff(double trc,long ri,long ci,long nn,long bc,long ipp)
{
  double c,pw,t;
  c = 0.0;
  
  pw = nodalval (nn,0);
  t = nodalval (nn,1);

  if((ri == 0) && (ci == 0))
    c = get_transmission_transcoeff_ww(pw,t,bc,ipp);// *scale_pw;//scaling
  if((ri == 0) && (ci == 1))
    c = 0.0;
  
  if((ri == 1) && (ci == 0))
    c = 0.0;
  if((ri == 1) && (ci == 1))
    c = get_transmission_transcoeff_tt(pw,t,bc,ipp);// *scale_t;//scaling

  c = c*trc;

  return (c);
}


/**
   function computes new transmission coefficient (for transmission_vector) 
   for boundary condition (third kind of boundary condition)

   @param trc     - prescribed transmission coefficient on the boundary
   @param ri      - row index
   @param ci      - column index
   @param nn      - number of node
   @param bc      - type of boundary condition
   @param ipp     - number of first integration point on element
*/
double con_hwf2mat::transmission_transcoeff(double trc,long ri,long ci,long nn,long bc,long ipp, int flag)
{
  double c,pw,t;
  c = 0.0;
  
  pw = nodalval (nn,0);
  t = nodalval (nn,1);

  if((ri == 0) && (ci == 0))
    c = get_transmission_transcoeff_ww(pw,t,bc,ipp,flag);// *scale_pw;//scaling
  if((ri == 0) && (ci == 1))
    c = 0.0;
  
  if((ri == 1) && (ci == 0))
    c = 0.0;
  if((ri == 1) && (ci == 1))
    c = get_transmission_transcoeff_tt(pw,t,bc,ipp);// *scale_t;//scaling
  
  c = c*trc;
  
  return (c);
}

/**
   function computes new nodal value (for transmission_vector) 
   for boundary condition (third kind of boundary condition)

   @param nodval     - prescribed nodal value
   @param trc2       - second prescribed transmission coefficient on the boundary, 
                       if is needed (for example heat radiation coef.)
   @param ri         - row index
   @param ci         - column index
   @param nn         - number of node
   @param bc         - type of boundary condition
   @param ipp        - number of first integration point on element
*/
double con_hwf2mat::transmission_nodval(double nodval,double trc2,long ri,long ci,long nn,long bc,long ipp)
{
  double c,pw,t;
  c = 0.0;
  
  pw = nodalval (nn,0);
  t = nodalval (nn,1);

  if((ri == 0) && (ci == 0))
    c = get_transmission_nodval_ww(nodval,pw,t,bc,ipp);// *scale_pw;//scaling
  if((ri == 0) && (ci == 1))
    c = 0.0;
  
  if((ri == 1) && (ci == 0))
    c = 0.0;
  if((ri == 1) && (ci == 1))
    c = get_transmission_nodval_tt(nodval,trc2,pw,t,bc,ipp);// *scale_t;//scaling

  return (c);
}


/**
   function computes flux (for transmission_vector) 
   for boundary condition (third kind of boundary condition)

   @param nodval     - prescribed nodal value
   @param trc2       - second prescribed transmission coefficient on the boundary, 
                       if is needed (for example heat radiation coef.)
   @param ri         - row index
   @param ci         - column index
   @param nn         - number of node
   @param bc         - type of boundary condition
   @param ipp        - number of first integration point on element
*/
double con_hwf2mat::transmission_flux(double nodval,double trc2,long ri,long ci,long nn,long bc,long ipp)
{
  double c,pw,t;
  c = 0.0;
  
  pw = nodalval (nn,0);
  t = nodalval (nn,1);

  if((ri == 0) && (ci == 0))
    c = get_transmission_flux_ww(nodval,pw,t,bc,ipp);// *scale_pw;//scaling
  if((ri == 0) && (ci == 1))
    c = 0.0;
  
  if((ri == 1) && (ci == 0))
    c = 0.0;
  if((ri == 1) && (ci == 1))
    c = get_transmission_flux_tt(nodval,trc2,pw,t,bc,ipp);// *scale_t;//scaling

  return (c);
}



/**
   function checks pore water pressure

   @param pw - pore water pressure
   @param t - temperature
   @param ipp - number of integration point
   
   
   TKr, 18.5.2005
*/
void con_hwf2mat::waterpress_check(double &pw,double t,long ipp)
{
  switch (model_type){
  case lewis_and_schrefler2hw:{//Lewis and Schrefler's book
    //this must be checked:
    if (pw >= 0.0) 
      pw = -1.0;
    
    //storing into gauss point
    Tm->ip[ipp].av[0] = pw;
    break;
  }
  case lewis_and_schrefler2hw_mefel:{//Lewis and Schrefler's book, saturation degree ind its derivative are obtained from mefel;
    break;
  }
  case van_genuchten2hw:{//partially saturated medium = Van Genuchten model
    break;
  }
  default:{
    print_err ("unknown model type is required", __FILE__, __LINE__, __func__);
    abort();
  }
  }
}

/**
   function corrects values of variables
   
   @param nv - array with variables
   
*/
void con_hwf2mat::values_correction (vector &nv)
{
  //  pore water pressure
  waterpress_check(nv[0],nv[1],0);
  
}


/**
   function creates capacity coefficient of a general material
   @param pw - pore water pressure
   @param t - temperature
   @param ipp - number of integration point
   
   @retval capww - capacity coefficient
*/
double con_hwf2mat::get_capww(double pw,double t,long ipp)
{
  double capww;
  double alpha,n,ks,sw,rhow,kw,dsw_dpw;

  //water pressure check
  waterpress_check(pw,t,ipp);

  alpha = get_alpha(pw,t,ipp);
  n = get_porosity(pw,t,ipp);
  ks = get_ks(pw,t,ipp);
  sw = get_sw(pw,t,ipp);
  rhow = get_rhow(t);
  kw = get_kw(pw,t,ipp);
  dsw_dpw = get_dsw_dpw(pw,t,ipp);

  if(compress == 1){
    //compressible grains:
    capww = (alpha - n)/ks*sw*(rhow*sw) + rhow*sw*n/kw;
    capww = capww + ((alpha - n)/ks*(rhow*sw*pw) + n*rhow)*dsw_dpw;
  }
  else{
    //incompressible grains:
    capww =  n*rhow*dsw_dpw;
  }
  
  return(capww);
}




/**
   function creates capacity coefficient of a general material
   @param pw - pore water pressure
   @param t - temperature
   @param ipp - number of integration point
   
   @retval capwt - capacity coefficient
*/
double con_hwf2mat::get_capwt(double pw,double t,long ipp)
{
  double capwt;
  double sw,alpha,n,ks,rhow,dsw_dt;
  double betasw;

  //water pressure check
  waterpress_check(pw,t,ipp);

  sw = get_sw(pw,t,ipp);
  alpha = get_alpha(pw,t,ipp);
  n = get_porosity(pw,t,ipp);
  ks = get_ks(pw,t,ipp);
  rhow = get_rhow(t);
  dsw_dt = get_dsw_dt(pw,t,ipp);
  betasw = get_betasw(pw,t,ipp);
  
  if(compress == 1){
    //compressible grains:
    capwt = ((alpha - n)/ks*(rhow*sw*pw) + n*rhow)*dsw_dt;
    capwt = capwt - betasw*rhow;
  }
  else{
    //incompressible grains:
    capwt = n*rhow*dsw_dt; 
  }
  
  return(capwt);
}



/**
   function creates conductivity coefficient of a general material
   @param pw - pore water pressure
   @param t - temperature
   @param ipp - number of integration point

   @retval kww - conductivity coefficient
*/
double con_hwf2mat::get_kww(double pw,double t,long ipp)
{
  double kww;
  double rhow,krw,muw,kintr;

  //water pressure check
  waterpress_check(pw,t,ipp);

  rhow = get_rhow(t);
  krw = get_krw(pw,t,ipp);
  muw = get_muw(t);
  kintr = get_kintr(pw,t,ipp);

  kww = rhow*kintr*krw/muw;

  return(kww);
}



/**
   function creates conductivity coefficient of a general material
   @param pw - pore water pressure
   @param t - temperature
   @param ipp - number of integration point

   @retval kwt - conductivity coefficient
*/
double con_hwf2mat::get_kwt(double pw,double t,long ipp)
{
  double kwt;
 
  kwt = 0.0;

  return(kwt);
}


/**
   function creates capacity coefficient of a general material
   @param pw - pore water pressure
   @param t - temperature
   @param ipp - number of integration point
   
   @retval captw - capacity coefficient
*/
double con_hwf2mat::get_captw(double pw,double t,long ipp)
{
  double captw;

  captw = 0.0;

  return(captw);
}



/**
   function creates capacity coefficient of a general material
   @param pw - pore water pressure
   @param t - temperature
   @param ipp - number of integration point
   
   @retval captt - capacity coefficient
*/
double con_hwf2mat::get_captt(double pw,double t,long ipp)
{
  double captt,rhocp;
   
  //water pressure check
  waterpress_check(pw,t,ipp);

  rhocp = get_rhocp(pw,t,ipp);

  captt = rhocp;

  return(captt);
}



/**
   function creates conductivity coefficient of a general material
   @param pw - pore water pressure
   @param t - temperature
   @param ipp - number of integration point
   
   @retval ktw - conductivity coefficient
*/
double con_hwf2mat::get_ktw(double pw,double t,long ipp)
{
  double ktw;

  ktw = 0.0;

  return(ktw);
}




/**
   function creates conductivity coefficient of a general material - first part
   @param pw - pore water pressure
   @param t - temperature
   @param ipp - number of integration point

   @retval ktt1 - conductivity coefficient
*/
double con_hwf2mat::get_ktt1(double pw,double t,long ipp)
{
  double lambdaeff,ktt1;
   
  //water pressure check
  waterpress_check(pw,t,ipp);

  lambdaeff = get_lambdaeff(pw,t,ipp);

  ktt1 = lambdaeff;

  return(ktt1);
}


/**
   function creates conductivity coefficient of a general material - second (A) part (convective term)
   @param pw - pore water pressure
   @param t - temperature
   @param ipp - number of integration point

   @retval ktt2a - conductivity coefficient
*/
double con_hwf2mat::get_ktt2a(double pw,double t,long ipp)
{
  double n,sw,cpw,rhow,krw,muw,ktt2a,kintr;
    
  //water pressure check
  waterpress_check(pw,t,ipp);

  n = get_porosity(pw,t,ipp);
  sw = get_sw(pw,t,ipp);
  rhow = get_rhow(t);
  cpw = get_cpw();
  krw = get_krw(pw,t,ipp);
  muw = get_muw(t);
  kintr = get_kintr(pw,t,ipp);

  ktt2a = n*sw*rhow*cpw*kintr*krw/muw;
  
  return(ktt2a);
}


/**
   function creates conductivity coefficient of a general material - second (B) part (convective term)
   @param pw - pore water pressure
   @param t - temperature
   @param ipp - number of integration point

   @retval ktt2c - conductivity coefficient
*/
double con_hwf2mat::get_ktt2c(double pw,double t,long ipp)
{
  double rhow,ktt2c;
  
  //water pressure check
  waterpress_check(pw,t,ipp);
  
  rhow = get_rhow(t);
  
  ktt2c = rhow;

  return(ktt2c);
}


/**
   function creates right-hand side coefficient of a general material for c medium
   @param pw - pore water pressure
   @param t - temperature
   @param ipp - number of integration point
*/

double con_hwf2mat::get_fw1(double pw,double t,long ipp)
{
  double fw1;
  double rhow,krw,muw,kintr;
  
  //water pressure check
  waterpress_check(pw,t,ipp);

  rhow = get_rhow(t);
  krw = get_krw(pw,t,ipp);
  muw = get_muw(t);
  kintr = get_kintr(pw,t,ipp);

  fw1 = rhow*kintr*krw*rhow/muw;
  
  return(fw1);
}


/**
   function creates right-hand side coefficient of a general material for t medium
   @param pw - pore water pressure
   @param t - temperature
   @param ipp - number of integration point
*/

double con_hwf2mat::get_ft1(double pw,double t,long ipp)
{
  double ft1;

  ft1 = 0.0;

  return(ft1);
}



/**
   function returns coefficient for righ-hand side of the general material
   @param pw - water pressure
   @param t - temperature
   @param ipp - number of integration point

   @retval fwu - volumetric strain rate effect on pore water pressure; only for partially coupled METR version

   22/05/2018, TKr
*/
double con_hwf2mat::get_fwu(double pw, double t, long ipp)
{
  double depsv_r,sw,fwu,alpha,dsr_depsv,n;
  
  fwu = 0.0;
  
  switch (model_type){
  case lewis_and_schrefler2hw:{
    break;
  }
  case lewis_and_schrefler2hw_mefel:{//Lewis and Schrefler's book
    n = get_porosity(pw,t,ipp);
    depsv_r = Tm->givenontransq(strain_vol_rate, ipp);            //actual rate of volumetric strain from MEFEL
    dsr_depsv = Tm->givenontransq(der_saturation_deg_depsv, ipp); //actual derivative of saturation degree with respect to volumetric strain
    Tm->ip[ipp].eqother[0] = dsr_depsv;
    Tm->ip[ipp].eqother[1] = depsv_r;

    sw = get_sw(pw,t,ipp);
    alpha = get_alpha(pw,t,ipp);
    fwu = -alpha*sw*depsv_r;//volumetric strain effect
    fwu = fwu -n*dsr_depsv*depsv_r;
    break;
  }
  case van_genuchten2hw:{
    break;
  }
  default:{
    print_err ("unknown model type is required", __FILE__, __LINE__, __func__);
    abort();
  }
  }    
  return(fwu);
}




/**
   function creates correct transfer coefficient on the boundary (transmission) for pw

   @param pw - pore water pressure
   @param t - temperature
   @param bc - type of boundary condition
   @param ipp - number of first integration point on element
*/
double con_hwf2mat::get_transmission_transcoeff_ww(double pw,double t,long bc,long ipp)
{
  double trc;
  
  //water pressure check
  waterpress_check(pw,t,ipp);

  //other conditions types will be added according to the neeed
  switch (bc){//type of prescribed variable
  case 30:{//pore water pressure

    trc = 1.0;
    
    break;
  }
  case 40:{//simulation of free soil surface
    if(pw < 0.0)
      trc = 0.0;
    else
      trc = 1.0e+18;
    break;
  }
  default:{
    print_err ("no real boundary condition is prescribed", __FILE__, __LINE__, __func__);
    exit(0);
  }
  }
  return(trc);
}



/**
   function creates correct transfer coefficient on the boundary (transmission) for pw

   @param pw - pore water pressure
   @param t - temperature
   @param bc - type of boundary condition
   @param ipp - number of first integration point on element
*/
double con_hwf2mat::get_transmission_transcoeff_ww(double pw,double t,long bc,long ipp,int flag)
{
  double trc;
  
  //water pressure check
  waterpress_check(pw,t,ipp);
  
  //other conditions types will be added according to the neeed
  switch (bc){//type of prescribed variable
  case 30:{//pore water pressure
    
    trc = 1.0;
    
    break;
  }
  case 33:{//pore water pressure - flux
    if(flag == 1)
      trc=0.0;//into right hand side (matrix)
    else{
      trc=1.0;//into left hand side (flux)
    }
    break;
  }
  case 40:{//simulation of free soil surface
    if(pw < 0.0)
      trc = 1.0e-18;
    else
      trc = 1.0e+18;
    break;
  }
  default:{
    print_err ("no real boundary condition is prescribed", __FILE__, __LINE__, __func__);
    exit(0);
  }
  }
  return(trc);
}

/**
   function creates correct new nodal value on the boundary (transmission) for pw

   @param bv - prescribed value near the boundary
   @param pw - actual pore water pressure on the boundary
   @param t - actual temperature on the boundary
   @param bc - type of boundary condition
   @param ipp - number of first integration point on element
*/
double con_hwf2mat::get_transmission_nodval_ww(double bv,double pw,double t,long bc,long ipp)
{
  double new_nodval;
  
  //water pressure check
  waterpress_check(pw,t,ipp);

  switch (bc){//type of prescribed variable
  case 30:{//pore water pressure
    new_nodval = bv;
    break;
  }
  case 33:{//pore water pressure - flux
    new_nodval = bv - pw;//minus sign
    break;
  }
  case 40:{//simulation of free soil surface
    new_nodval = bv;
    break;
  }
  default:{
    print_err ("no real boundary condition is prescribed", __FILE__, __LINE__, __func__);
    exit(0);
  }
  }
  return(new_nodval);
}



/**
   function creates flux on the boundary (transmission - convective mass transfer) for pw

   @param bv - prescribed value near the boundary
   @param pw - actual pore water pressure on the boundary
   @param t - actual temperature on the boundary
   @param bc - type of boundary condition
   @param ipp - number of first integration point on element
*/
double con_hwf2mat::get_transmission_flux_ww(double bv,double pw,double t,long bc,long ipp)
{
  double flux,trc;
  
  //water pressure check
  waterpress_check(pw,t,ipp);

  switch (bc){//type of prescribed variable
  case 30:{//pore water pressure
    flux = bv - pw;//minus sign
    break;
  }
  case 33:{//pore water pressure - flux
    flux = bv - pw;//minus sign
    break;
  }
  case 40:{//simulation of free soil surface
    if(pw >= 0.0)
      trc = 1.0e-06;
    else
      trc = 1.0e+09;
    flux = trc*(bv - pw);//minus sign
    break;
  }
  default:{
    print_err ("no real boundary condition is prescribed", __FILE__, __LINE__, __func__);
    exit(0);
  }
  }

  return(flux);
}


/**
   function creates correct transfer coefficient on the boundary (transmission) for t medium

   @param pw - pore water pressure
   @param t - temperature
   @param bc - type of boundary condition
   @param ipp - number of first integration point on element
*/
double con_hwf2mat::get_transmission_transcoeff_tt(double pw,double t,long bc,long ipp)
{
  double trc;

  //water pressure check
  waterpress_check(pw,t,ipp);

  switch (bc){//type of prescribed variable
  case 30:{//heat transmission
    trc = 1.0;
    break;
  }
  default:{
    print_err ("no real boundary condition is prescribed", __FILE__, __LINE__, __func__);
    exit(0);
  }
  }
  
  return(trc);
}




/**
   function creates correct new nodal value on the boundary (transmission) for t medium

   @param bv - value of prescribed value near the boundary
   @param trr - trr coefficient
   @param pw - actual pore water pressure on the boundary
   @param t - actual temperature on the boundary
   @param bc - type of boundary condition
   @param ipp - number of first integration point on element
*/
double con_hwf2mat::get_transmission_nodval_tt(double bv,double trr,double pw,double t,long bc,long ipp)
{
  double new_nodval;

  //water pressure check
  waterpress_check(pw,t,ipp);

  switch (bc){//type of prescribed variable
  case 30:{//heat transmission
    new_nodval = bv;
    break;
  }
  default:{
    print_err ("no real boundary condition is prescribed", __FILE__, __LINE__, __func__);
    exit(0);
  }
  }

  return(new_nodval);
}


/**
   function creates flux on the boundary (transmission - convective mass transfer) for c medium

   @param bv - prescribed value near the boundary
   @param trr - trr coefficient
   @param pw - actual pore water pressure on the boundary
   @param t - actual temperature on the boundary
   @param bc - type of boundary condition
   @param ipp - number of first integration point on element
*/
double con_hwf2mat::get_transmission_flux_tt(double bv,double trr,double pw,double t,long bc,long ipp)
{
  double flux;

  //water pressure check
  waterpress_check(pw,t,ipp);

  switch (bc){//type of prescribed variable
  case 30:{//heat transmission - boundary flux
    flux = (bv - t);//minus sign
    break;
  }
  default:{
    print_err ("no real boundary condition is prescribed", __FILE__, __LINE__, __func__);
    exit(0);
  }
  }

  return(flux);
}



/**
   function computes all variables in nodes
   @param compother - number of other components
   @param ipp - first integration point on element
   @param pw - pore water pressure on actual node
   @param t - temperature on actual node
*/

double con_hwf2mat::get_othervalue(long compother,long ipp,double *r)
{
  double other;

  switch (compother){
  case 0:{//capillary pressure
    other = r[0];
    break;
  }
  case 1:{//temperature in deg. C
    other = r[1] - 273.15;
    break;
  }
  case 2:{//relative humidity
    other = 0.0;//get_rh(r[0],r[2]);//not completed
    break;
  }
  case 3:{//saturation
    other = get_sw(r[0],r[1],ipp);
    break;
  }
  case 4:{//liquid water pressure
    other = r[0];
    break;
  }
  case 5:{//moisture content
    other = get_w(r[0],r[1],ipp);
    break;
  }    
  case 6:{//dsw_dpw
    other = get_dsw_dpw(r[0],r[1],ipp);
    break;
  }    
  case 7:{//dsw_dt
    other = get_dsw_dt(r[0],r[1],ipp);
    break;
  }    
  case 8:{//depsv
    other = Tm->ip[ipp].eqother[1];
    break;
  }   
  default:{
    print_err ("unknown type of component is required in function", __FILE__, __LINE__, __func__);
    abort();
  }
  }
  return (other);

}

/**
     function prints names of all variables in nodes
     @param out - output file
     @param compother - number of other components
*/
void con_hwf2mat::print_othervalue_name(FILE *out,long compother)
{
  switch (compother){
  case 0:{//capillary pressure
    fprintf (out,"Capilary pressure (Pa)        ");
    break;
  }
  case 1:{//temperature
    fprintf (out,"Temperature (K)               ");
    break;
  }
  case 2:{//relative humidity
    fprintf (out,"Relative humidity ()          ");
    break;
  }
  case 3:{//saturation
    fprintf (out,"Degree of saturation ()       ");
    break;
  }
  case 4:{//liquid water pressure
    fprintf (out,"Pore water pressure (Pa)      ");
    break;
  }
  case 5:{//moisture content
    fprintf (out,"Moisture (water) content (kg/kg)      ");
    break;
  }    
  case 6:{//dS_dpw
    fprintf (out,"dS_dpw Derivative of Saturation degree with respect to pore water pressure      ");
    break;
  }    
  case 7:{//dS_dt
    fprintf (out,"dS_dt Derivative of Saturation degree with respect to temperature      ");
    break;
  }    
  default:{
  case 8:{//depsv
    fprintf (out,"Volumetric strain rate (-)       ");
    break;
  }    
    print_err ("unknown type of component is required in function", __FILE__, __LINE__, __func__);
    abort();
  }
  }
}



/**
  This function updates values in the other array reached in the previous equlibrium state to
  values reached in the new actual equilibrium state.

  @param ipp - integration point number in the mechmat ip array.

   23/05/2016, TKr
*/
void con_hwf2mat::updateval (long ipp)
{
}



/**
   This function initializes material model data
   with respect of consistency parameter gamma.

   @param ipp - integration point number

   23/05/2016, TKr
*/
void con_hwf2mat::initval(long ipp)
{
}



/**
  The function returns ordered dof names of primary unknowns 
  required by the model.

  @param dofname   - array of uknown name for particular nodal dofs (output)
                     dofname[i] = name of i-th nodal unknown (for names see aliast.h - enum namevart)
  @param ntm       - number of transported media = number of nodal dof = length of array dofname

  Created by Tomas Krejci according to Tomas Koudelka, 23/05/2016
*/
void con_hwf2mat::give_dof_names(namevart *dofname, long ntm)
{
  if (ntm < 1)
  {
    print_err("the model defines %ld unknowns but number of transported media is %ld", 
              __FILE__, __LINE__, __func__, 1, ntm);
    abort();
  }
  dofname[0] = trf_water_press;
  dofname[1] = trf_temperature;
}


/**
   function returns temperature

   @param ipp - integration point number

   @retval t - temperature

   16/07/2018, TKr
*/
double con_hwf2mat::give_temperature(long ipp)
{
  double t;

  t = Tm->ip[ipp].av[1];

  return(t);
}


/**
   function returns water pressure

   @param ipp - integration point number

   @retval pw - pore water pressure

   27/05/2016, TKr
*/
double con_hwf2mat::give_water_pressure(long ipp)
{
  double pw;

  pw = Tm->ip[ipp].av[0];

  return(pw);
}

/**
   function returns pore pressure

   @param ipp - integration point number

   @retval pp - pore pressure

   27/05/2016, TKr
*/
double con_hwf2mat::give_pore_pressure(long ipp)
{
  double pp,pw;

  pw = Tm->ip[ipp].av[0];

  pp = -pw;

  return(pp);
}


/**
   function computes suction stress s = -(pg - pw) = -pc;
   @param ipp - integration point number

   @retval suction - suction stress [Pa]

   27/05/2016, TKr
*/
double con_hwf2mat::give_suction(long ipp)
{
  double pw,suction;
  
  pw = Tm->ip[ipp].av[0];
  
  switch (model_type){
  case lewis_and_schrefler2hw:{
    suction = -pw; //this is correct for Lewis and Schrefler's notation
    break;
  }
  case lewis_and_schrefler2hw_mefel:{//Lewis and Schrefler's book, saturation degree ind its derivative are obtained from mefel
    suction = pw;
    suction = suction/mefel_units;//corrected units for mefel //basic units = Pa
    break;
  }
  case van_genuchten2hw:{//partially saturated medium = Van Genuchten model
    pw = pw/mefel_units;
    suction = pw;//this is correct, because pore water pressure is negative
    break;
  }
  default:{
    print_err ("unknown model type is required", __FILE__, __LINE__, __func__);
    abort();
  }
  } 
  return(suction);
}


/**
   function gives saturation degree
   @param ipp - integration point number

   @retval saturation degree []

   27/05/2016, TKr
*/
double con_hwf2mat::give_saturation_degree(long ipp)
{
  double pw,t,s;
  
  pw = Tm->ip[ipp].av[0];
  t = Tm->ip[ipp].av[1];
  

  s = get_sw(pw,t,ipp);
  
  return(s);
}

/**
   function returns porosity

   @retval por - porosity

   3/10/2010, TKr
*/
double con_hwf2mat::get_porosity(double pw, double t,long ipp)
{
  double por;

  switch (model_type){
  case lewis_and_schrefler2hw:{//Lewis and Schrefler's book
    por = phi0; //constant value
    break;
  }
  case lewis_and_schrefler2hw_mefel:{//Lewis and Schrefler's book
    if(Tm->nontransq != NULL){
      por = Tm->givenontransq(porosity, ipp);// from mefel
    }
    else
      por = phi0; //constant value for testing
    break;
  }
  case van_genuchten2hw:{//partially saturated medium = Van Genuchten model
    por = phi0; //constant value
    break;
  }
  default:{
    print_err ("unknown model type is required", __FILE__, __LINE__, __func__);
    abort();
  }
  } 

  return por;
}


/**
   function computes water content w [kg/kg]
   @param pw - water capillary pressure
   @param t - temperature
   

   @retval w - computes water content w [kg/kg]
*/
double con_hwf2mat::get_w(double pw,double t,long ipp)
{
  double w,n,s,rhow;

  n = get_porosity(pw,t,ipp);
  s = get_sw(pw,t,ipp);
  rhow = get_rhow(t);

  w = (n*s*rhow)/(1.0 - n)/rhos0;

  return(w);
}




/**
  The funtion marks required non-transport quantities in the array antq.

  @param antq - array with flags for used material types
                antq[i] = 1 => quantity type nontransquant(i+1) is required
                antq[i] = 0 => quantity type nontransquant(i+1) is not required

  @return The function does not return anything, but it may change content of antq array.
  
  27/05/2016, TKr
*/
void con_hwf2mat::give_reqntq(long *antq)
{
  switch (model_type){
  case lewis_and_schrefler2hw:{//Lewis and Schrefler's book
    break;
  }
  case lewis_and_schrefler2hw_mefel:{//Lewis and Schrefler's book, saturation degree ind its derivative are obtained from mefel;
    antq[saturation_deg-1] = 1;
    antq[der_saturation_deg-1] = 1;
    antq[der_saturation_deg_depsv-1] = 1;
    antq[porosity-1] = 1;
    antq[der_saturation_deg_dtemp-1] = 1;
    antq[strain_vol_rate-1] = 1;
    break;
  }  
  case van_genuchten2hw:{//partially saturated medium = Van Genuchten model
    break;
  }
  default:{
    print_err("unknown model type is required", __FILE__, __LINE__, __func__);
  }
  } 
}
