/*
    File:            van_genuchten.cpp
    Author:          Tomas Krejci, 13/04/2018
    Purpose:         retention curve accorging to Van_Genuchten model
    sources:                              
		     M. Th. van Genuchten, A closed equation for predicting the hydraulic conductivity of
		     unsaturated soils, Journal Soil Science Society of America 44 (1980), 892-898;
		     pore water pressure (suction) is assumed in kPa (kilo Pascals)
*/ 

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "van_genuchten.h"

van_genuchten_reten::van_genuchten_reten()
{ 
  ssat = 1.0;      //saturation degree [-]
  sirr = 0.2;      //irreversible saturation degree [-]
  ksat = 0.1;      //saturation permeability [??]
  gamaw = 10000.0; //water weigth [N/m^3]
  delta = 20.0;    //parameter in m^(-1); delta = 20.0 [1/m]; delta = 0.2 [1/cm]
  expn = 2.0;      //parameter
}

van_genuchten_reten::~van_genuchten_reten()
{}


/**
   function reads parameters
   
   @param in - input file

   TKr 3/04/2018
*/
void van_genuchten_reten::read(XFILE *in)
{
  xfscanf (in,"%lf %lf %lf %lf %lf", &ssat, &sirr, &expn, &ksat, &delta);
}

/**
   function reads parameters
   
   @param in - input file

   TKr 3/04/2018
*/
void van_genuchten_reten::print(FILE *out)
{
  fprintf (out,"\n%lf %lf %lf %lf %lf\n", ssat, sirr, expn, ksat, delta);
}

/**
   function computes saturation degree

   @param pw - pore water pressure
   @retval sw - saturation degree
   
   TKr 3/04/2018
*/
double van_genuchten_reten::sw(double pw)
{
  double sw,hp,expm,theta;
  
  if(pw >= 0.0)
    pw = 0.0;//for positive pressure
  hp = pw/gamaw;
  expm = 1.0 - 1.0/expn;
  theta = pow(1+(pow(delta*fabs(hp),expn)),-expm);
  sw = sirr + (ssat - sirr)*theta;
  //sw = 1.0;//debug??!!
  return(sw);
}


/**
   function computes partial derivative of degree of saturation with respect to pw, specific water content
   
   @param pw - water pressure
   @retval dsw_dpw - partial derivative of degree of saturation with respect to pw
   
   TKr 3/04/2018
*/
double van_genuchten_reten::dsw_dpw(double pw)
{
  double dsw_dpw,hp,expm;
  
  if(pw >= 0.0)
    pw = 0.0;//for positive pressure
  hp = pw/gamaw;
  expm = 1.0 - 1.0/expn;
  dsw_dpw = (expm*expn*delta*(ssat - sirr)*pow(delta*fabs(hp),(expn-1)))/(pow(1+(pow(delta*fabs(hp),expn)),(expm+1)))/gamaw;
  //dsw_dpw = 1.7e-6;//debug??!!
  return(dsw_dpw);
}


/**
   function computes partial derivative of degree of saturation with respect to t

   @param pw - water pressure
   @retval dsw_dt - partial derivative of degree of saturation with respect to t

   TKr 3/04/2018
*/
double van_genuchten_reten::dsw_dt(double pw)
{
  double dsw_dt;
  
  dsw_dt = 0.0;
  
  return(dsw_dt);
}


/**
   function computes water relative permeability

   @param pw - water pressure
   @retval krw - water relative permeability

   TKr 3/04/2018
*/
double van_genuchten_reten::krw(double pw)
{
  double krw,hp,expm;

  expm = 1.0 - 1.0/expn;
  hp = pw/gamaw;
  krw = pow(1.0-pow(delta*fabs(hp),expn)*pow(1.0+pow(delta*fabs(hp),expn),-expm),2.0)/
    pow(1.0+pow(delta*fabs(hp),expn),(expm/2.0));
  //krw = 1.0/(pow(1.0+(pow(delta*fabs(hp),expn)),(3.0/expm)));//alternative expression according to Fatt and klikof
  //krw = 1.0/(pow(1.0+(pow(delta*fabs(hp),expn)),(1.0/expm)));//alternative expression according Warrick
  krw = 1.0;//debug??!!
  return(krw);
}


/**
   function computes water conductivity of porous material 

   @retval k - conductivity

   13/04/2018, TKr
*/
double van_genuchten_reten::get_k(double pw)
{
  double k;

  k = ksat/86400.0/gamaw;

  return(k);
}
